/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "Aria.h"


/*
  This is an example to show how to use ArSocket to create a client and 
  a server. This file is the client side. All this program does is connect
  to a server on the host 'localhost' on port 7777. It waits for the server
  to say hello, then it says hello in reponse. It then closes the socket and
  exits.

  To run this, first run the server program then run the client program.
  They should be run in seperate windows so that the output from both
  programs is not confused.

  Look at examples/serverHelloWorld.cpp for the server code. 
*/


int main()
{
  // The string to send to the server. Done here as a char array so that its
  // easier to figure out its length.
  char *strToSend="Hello Server";
  // The buffer in which to recieve the hello from the server
  char buff[100];
  // The size of the string the server sent
  size_t strSize;
  // The socket object
  ArSocket sock;

  // Initialize Aria. For Windows, this absolutely must be done. Because
  // Windows does not initialize the socket layer for each program. Each
  // program must initialize the sockets itself.
  Aria::init();

  // Lets connect to the server
  if (sock.connect("localhost", 7777, ArSocket::TCP))
    printf("Connected to server\n");
  else
  {
    printf("Failed to connect to server: %s\n", sock.getErrorStr().c_str());
    return(-1);
  }

  // Lets wait for the server to say hello to us.
  strSize=sock.read(buff, sizeof(buff));

  // If the amount read is 0 or less, its an error condition.
  if (strSize > 0)
  {
    // Terminate the string with a NULL character.
    buff[strSize]='\0';
    printf("Server said: %s\n", buff);
  }
  else
  {
    printf("Error in waiting/reading the hello from the server\n");
    return(-1);
  }

  // Lets send the string 'Hello Server' to the server. The write should
  // return the same number of bytes that we told it to write. Otherwise,
  // its an error condition.
  if (sock.write(strToSend, strlen(strToSend)) == strlen(strToSend))
    printf("Said hello to the server\n");
  else
  {
    printf("Error in sending hello string to the server\n");
    return(-1);
  }

  // Now lets close the connection to the server
  sock.close();
  printf("Socket to server closed\n");

  // Uninitialize Aria
  Aria::uninit();

  // All done
  return(0);
}
