/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "Aria.h"


/*
  This is an example to show how to use ArSocket to create a client and 
  a server. This file is the server side. All this program does is open
  a server port on the host 'localhost' and on port 7777. It waits for the
  client to connect and says hello to the client. It then waits for the
  client to say hello and then exits.

  To run this, first run the server program then run the client program.
  They should be run in seperate windows so that the output from both
  programs is not confused.

  Look at examples/clientHelloWorld.cpp for the client code. 
*/


int main()
{
  // The string to send to the client. Done here as a char array so that its
  // easier to figure out its length.
  char *strToSend="Hello Client";
  // The buffer in which to recieve the hello from the client
  char buff[100];
  // The size of the string the client sent
  size_t strSize;
  // The socket object
  ArSocket serverSock, clientSock;

  // Initialize Aria. For Windows, this absolutely must be done. Because
  // Windows does not initialize the socket layer for each program. Each
  // program must initialize the sockets itself.
  Aria::init();

  // Lets open the server socket
  if (serverSock.open(7777, ArSocket::TCP))
    printf("Opened the server port\n");
  else
  {
    printf("Failed to open the server port: %s\n",
	   serverSock.getErrorStr().c_str());
    return(-1);
  }

  // Lets wait for the client to connect to us.
  if (serverSock.accept(&clientSock))
    printf("Client has connected\n");
  else
    printf("Error in accepting a connection from the client: %s\n",
	   serverSock.getErrorStr().c_str());

  // Lets send the string 'Hello Client' to the client. The write should
  // return the same number of bytes that we told it to write. Otherwise,
  // its an error condition.
  if (clientSock.write(strToSend, strlen(strToSend)) == strlen(strToSend))
    printf("Said hello to the client\n");
  else
  {
    printf("Error in sending hello string to the client\n");
    return(-1);
  }

  // Lets wait for the client to say hello to us.
  strSize=clientSock.read(buff, sizeof(buff));

  // If the amount read is 0 or less, its an error condition.
  if (strSize > 0)
  {
    // Terminate the string with a NULL character.
    buff[strSize]='\0';
    printf("Client said: %s\n", buff);
  }
  else
  {
    printf("Error in waiting/reading the hello from the client\n");
    return(-1);
  }

  // Now lets close the connection to the client
  clientSock.close();
  printf("Socket to client closed\n");
  // And lets close the server port
  serverSock.close();
  printf("Server socket closed\n");

  printf("%d\n", clientSock.getFD());

  // Uninitialize Aria
  Aria::uninit();

  // All done
  return(0);
}
