/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "Aria.h"


/*
  This is a simple program that loads the module simpleMod, which is
  defined in simpleMod.cpp. This module is a Hello World example. The
  program simply calls ArModuleLoader::load() with a short message
  argument and ArModuleLoader::reload with no argument and finally
  calls ArModuleLoader::close(). The return status of the load(),
  reload(), and close() are checked and printed out.

  See simpleMod.cpp.
  See ArModuleLoader in the reference manual.  
*/


void printStatus(ArModuleLoader::Status status)
{
  if (status == ArModuleLoader::STATUS_ALREADY_LOADED)
    ArLog::log(ArLog::Terse, "Module already loaded\n");
  else if (status == ArModuleLoader::STATUS_FAILED_OPEN)
    ArLog::log(ArLog::Terse, "Failed to find or open the simpleMod module\n");
  else if (status == ArModuleLoader::STATUS_INVALID)
    ArLog::log(ArLog::Terse, "Invalid file\n");
  else if (status == ArModuleLoader::STATUS_INIT_FAILED)
    ArLog::log(ArLog::Terse, "Module Init failed\n");
  else if (status == ArModuleLoader::STATUS_SUCCESS)
    ArLog::log(ArLog::Terse, "Module succedded\n");
  else if (status == ArModuleLoader::STATUS_EXIT_FAILED)
    ArLog::log(ArLog::Terse, "Module exit sequence failed\n");
  else if (status == ArModuleLoader::STATUS_NOT_FOUND)
    ArLog::log(ArLog::Terse, "Module not found\n");
  else
    ArLog::log(ArLog::Terse, "Module returned unknown STATUS\n");
}

int main()
{
  ArModuleLoader::Status status;
  ArRobot robot;
  std::string str;

  Aria::init();
  

  ArLog::log(ArLog::Terse, "Loading the module with a string argument.\n");
  status=ArModuleLoader::load("./simpleMod", &robot, (char *)"You've loaded a module!");
  printStatus(status);
  ArLog::log(ArLog::Terse, "Reloading the module with no argument.\n");
  status=ArModuleLoader::reload("./simpleMod", &robot);
  printStatus(status);

  ArLog::log(ArLog::Terse, "Closing the module.\n");
  status=ArModuleLoader::close("./simpleMod");
  printStatus(status);

  Aria::shutdown();
  return(0);
}
