/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARCOMMANDS_H
#define ARCOMMANDS_H

/// A class with an enum of the commands that can be sent to the robot
/**
   A class with an enum of the commands that can be sent to the robot, see the 
   operations manual for more detailed descriptions.
*/
// TODO find out about the commented out ones
class ArCommands
{
public:
  enum Commands { 
  PULSE = 0, ///< none, keep alive command, so watchdog doesn't trigger
  OPEN = 1, ///< none, sent after connection to initiate connection
  CLOSE = 2, ///< none, sent to close the connection to the robot
  POLLING = 3, ///< string, string that sets sonar polling sequence
  ENABLE = 4, ///< int, enable (1) or disable (0) motors
  SETA = 5, ///< int, sets translational accel (+) or decel (-) (mm/sec/sec)
  SETV = 6, ///< int, sets maximum velocity (mm/sec)
  SETO = 7, ///< int, resets robots origin back to 0, 0, 0
  MOVE = 8, ///< int, translational move (mm)
  ROTATE = 9, ///< int, set rotational velocity, duplicate of RVEL (deg/sec)
  SETRV = 10, ///< int, sets the maximum rotational velocity (deg/sec)
  VEL = 11, ///< int, set the translational velocity (mm/sec)
  HEAD = 12, ///< int, turn to absolute heading 0-359 (degrees)
  DHEAD = 13, ///< int, turn relative to current heading (degrees)
  //DROTATE = 14, does not really exist
  SAY = 15, /**< string, makes the robot beep.
	     up to 20 pairs of duration (20 ms incrs) and tones (halfcycle) */
  // VISION = 16, no clue
  // VWINDOW = 17, no clue what this does 
  CONFIG = 18, ///< int, request configuration packet
  ENCODER = 19, ///< int, > 0 to request continous stream of packets, 0 to stop
  RVEL = 21, ///< int, set rotational velocity (deg/sec)
  DCHEAD = 22, ///< int, colbert relative heading setpoint (degrees)
  SETRA = 23, ///< int, sets rotational accel(+) or decel(-) (deg/sec)
  SONAR = 28, ///< int, enable (1) or disable (0) sonar 
  STOP = 29, ///< int, stops the robot
  DIGOUT = 30, ///< int, sets the digout lines
  //TIMER = 31, ... no clue about this one
  VEL2 = 32, /**< 2bytes, independent wheel velocities,
		first byte = right, second = left */
  GRIPPER = 33, ///< int, gripper server command, see gripper manual for detail
  //KICK = 34, um...
  ADSEL = 35, ///< int, select the port given as argument
  GRIPPERVAL = 36, ///< p2 gripper server value, see gripper manual for details
  GRIPPERPACREQUEST = 37, ///< p2 gripper packet request
  IOREQUEST = 40, ///< request iopackets from p2os
  PTUPOS = 41, ///< most-sig byte is port number, least-sig byte is pulse width
  TTY2 = 42, ///< string, send string argument to serial dev connected to aux 
  GETAUX = 43, ///< int, requests 1-200 bytes from aux serial channel, 0 flush
  BUMPSTALL = 44, /**< int, stop and register a stall if front (1), rear (2),
		   or both (3) bump rings are triggered, Off (default) is 0 */
  TCM2 = 45, ///< TCM2 module commands, see p2 tcm2 manual for details 
  JOYDRIVE = 47, /**< Command to tell p2os to drive with the joystick 
		    plugged into the robot */
  ESTOP = 55, ///< none, emergency stop, overrides decel
  ESTALL = 56, // TODO ///< 
  // these are SIM specific
  LOADPARAM = 61, 
   ///< string, Sim Specific, causes the sim to load the given param file 
  ENDSIM = 62, ///< none, Sim Specific, causes the simulator to close and exit
  LOADWORLD = 63, ///< string, Sim Specific, causes the sim to load given world
  STEP = 64, ///< none, Sim Specific, single step mode
  // for calibrating the compass
  CALCOMP = 65, ///< int, commands for calibrating compass, see compass manual
  // back to SIM specific
  SETSIMORIGINX = 66, ///< int, sets the X origin in the simulator
  SETSIMORIGINY = 67, ///< int, sets the Y origin in the simulator
  SETSIMORIGINTH = 68, ///< int, sets the heading at origin in the simulator
  RESETSIMTOORIGIN = 69, ///< int, resets the sim robots poseiton to origin
  // AmigoBot specific
  SOUND = 90, ///< int, AmigoBot specific, plays sound with given number
  PLAYLIST = 91, /**< int, AmigoBot specific, requests name of sound, 
		    0 for all, otherwise for specific sound */
  SOUNDTOG = 92 ///< int, AmigoBot specific, enable(1) or diable(0) sound
  };
};

#endif // ARCOMMANDS_H


