/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARLOG_H
#define ARLOG_H

#ifdef linux
#include <stdio.h>
#endif
#include <string>
#include "ariaTypedefs.h"
#include "ArMutex.h"


/// Logging utility class
/**
   ArLog is a utility class to log all messages from Aria to a choosen
   destintation. Messages can be logged to stdout, stderr, a file, and
   turned off completely. Logging by default is set to stdout. The level
   of logging can be changed as well. Allowed levels are Terse, Normal,
   and Verbose. By default the level is set to Normal.
*/
class ArLog
{
public:

  typedef enum {
    StdOut, ///< Use stdout for logging
    StdErr, ///< Use stderr for logging
    File, ///< Use a file for logging
    Colbert, ///< Use a Colbert stream for logging
    None ///< Disable logging
  } LogType;
  typedef enum {
    Terse, ///< Use terse logging
    Normal, ///< Use normal logging
    Verbose ///< Use verbose logging
  } LogLevel;

  /// Log a message
  AREXPORT static void log(LogLevel level, char *str, ...);
  /// Initialize the logging utility
  AREXPORT static bool init(LogType type, LogLevel level,
			    const char *fileName="");
  /// Close the logging utility
  AREXPORT static void close();

  // Do not use this unless you know what you are doing...
  AREXPORT static void logNoLock(LogLevel level, char *str, ...);

  // We use this to print to a Colbert stream, if available
  AREXPORT static void (* colbertPrint)(int i, char *str);

protected:

  static ArLog *ourLog;
  static ArMutex ourMutex;
  static LogType ourType;
  static LogLevel ourLevel;
  static FILE *ourFP;
  static int ourColbertStream;
  static std::string ourFileName;
};


#endif
