/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef _ARPERIODICTASK_H
#define _ARPERIODICTASK_H


#include "ariaTypedefs.h"


/// A periodic task base class
/**
   This class is a base class for running periodic tasks that need to be
   run in their own thread. Since the tasks are periodic and dont need to
   be run constantly, they dont need their own thread all the time. Since
   threads can be a restricted resource, there is this periodic task. The
   user simply needs to derive their own class from ArPeriodicTask and
   override the runTask() function. The function is guarenteed to be run
   in its own thread. It can run as long as it wants and when its done
   the thread will be released to be used by another ArPeriodicTask that
   may need it. To run the task, simply call invoke() on it. The invoke()
   can be called multiple times.

   See also ArTaskPool.
*/
class ArPeriodicTask
{
public:

  /// Constructor
  AREXPORT ArPeriodicTask();

  /// Destructor
  AREXPORT virtual ~ArPeriodicTask();

  /// Invoke the task
  AREXPORT virtual bool invoke();

  /// Function to override for the task
  AREXPORT virtual void runTask() = 0;

  /// Function to set the task's running variable
  AREXPORT void setRunning(bool yesno);

  /// Function to tell if the task is running
  AREXPORT bool isRunning() { return myIsRunning; }

protected:

  bool myIsRunning;
  ArMutex myMutex;

};


#endif // _ARPERIODICTASK_H
