/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARRANGEDEVICETHREADED_H
#define ARRANGEDEVICETHREADED_H

#include "ariaTypedefs.h"
#include "ArRangeDevice.h"
#include "ArASyncTask.h"

/// A range device which can run in its own thread
class ArRangeDeviceThreaded : public ArRangeDevice, public ArASyncTask
{
public:
  /// Constructor
  AREXPORT ArRangeDeviceThreaded(size_t currentBufferSize, 
				 size_t cumulativeBufferSize,
				 const char *name, unsigned int maxRange);
  /// Destructor
  AREXPORT virtual ~ArRangeDeviceThreaded(void);
  AREXPORT virtual void * runThread(void *arg) = 0;
  /// Runs the device in this thread
  /**
     Runs the device in the calling thread.
     @see ArThread::stopRunning
  */
  AREXPORT void run(void) { runInThisThread(); }
  /// Runs the device in its own thread
  /**
     Runs the device in its own thread.
     @see ArThread::stopRunning
  */
  AREXPORT void runAsync(void) { create(false); }
  AREXPORT virtual int lockDevice(void) { return lock(); }
  AREXPORT virtual int tryLockDevice(void) { return tryLock(); }
  AREXPORT virtual int unlockDevice(void) { return unlock(); }
  
};

#endif // ARRANGEDEVICETHREADED_H
