/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARROBOTPARAMFILE_H
#define ARROBOTPARAMFILE_H


#include "ariaUtil.h"
#include "ArPreferences.h"


class ArRobotParamFile : public ArPreferences
{
public:

  // Section names
  enum { General, ConvFactors, Accessories, Sonar, Laser, LASTSECTION };

  // Preference names
  enum
  {
    // General parameters
    Class, Subclass, RobotRadius, RobotDiagonal, Holonomic, MaxRVelocity,
    MaxVelocity, HasMoveCommand, RequestIOPackets,

    // Conversion factors
    AngleConvFactor, DistConvFactor, VelConvFactor, RangeConvFactor,
    DiffConvFactor, Vel2Divisor,

    // Accessories
    TableSensingIR, NewTableSensingIR, FrontBumpers, NumFrontBumpers,
    RearBumpers, NumRearBumpers,

    // Sonar parameters
    SonarNum, SonarUnit,

    LaserPossessed, LaserPort, LaserPowerControlled, 
    LaserFlipped, LaserX, LaserY,
    LASTPREF
  };

  static const char *ourClassValues[];
  static const char *ourSubclassValues[];
  static ArPrefSection ourSections[];
  static ArPref ourPrefs[];

  AREXPORT ArRobotParamFile(const char *dir, const char *file);
  AREXPORT virtual ~ArRobotParamFile();

  AREXPORT virtual void setDefaults() = 0;

  AREXPORT virtual void init();

  AREXPORT virtual std::string getHeader();
  AREXPORT virtual std::string getDir();
  AREXPORT virtual std::string getFile();

protected:

  AREXPORT virtual bool findDir(const char *fileName);
  AREXPORT virtual bool findFile(const char *fileName);
  AREXPORT virtual bool handleListSectLine(const char *section,
					   const char *line);
  AREXPORT virtual void extraSave(FILE *fp);

  std::string myDir;
  std::string myFile;
  std::string myTmpFile;

private:

  // Hide the default constructor.
  AREXPORT ArRobotParamFile();

};


#endif // ARROBOTPARAMFILE_H
