/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARSONARDEVICE_H
#define ARSONARDEVICE_H

#include "ariaTypedefs.h"
#include "ArRangeDevice.h"
#include "ArFunctor.h"

#include "ArRobot.h"

/// A class for keeping track of sonar
/** 
    This class is for keeping a sonar history, and using that for obstacle 
    avoidance and displays and what not
*/
class ArSonarDevice : public ArRangeDevice
{
public:
  /// Constructor
  AREXPORT ArSonarDevice(size_t currentBufferSize = 24, 
			 size_t cumulativeBufferSize = 64, 
			 const char * name = "sonar");
  /// Destructor
  AREXPORT ~ArSonarDevice(void);
  /// Grabs the new readigns from the robot and adds them to the buffers
  AREXPORT void processReadings(void);
  /// Sets the robot pointer, also attaches its process function to the
  /// sensorInterp of the robot
  AREXPORT virtual void setRobot(ArRobot *robot);

  /// Adds sonar readings to the current and cumulative buffers
  /// Overrides the ArRangeDevice default action
  AREXPORT virtual void addReading(double x, double y);

  /// Maximum range for a reading to be added to the cumulative buffer (mm)
  AREXPORT void setCumulativeMaxRange(double r) { myCumulativeMaxRange = r; }
protected:
  ArFunctorC<ArSonarDevice> myProcessCB;
  double myCumulativeMaxRange;	// how close the reading must be
  double myFilterNearDist;	// we throw out cumulative readings this close to current one
  double myFilterFarDist;	// throw out cumulative readings beyond this far from robot
};


#endif // ARSONARDEVICE_H
