/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef _ARTASKPOOL_H
#define _ARTASKPOOL_H


#include <list>
#include <set>
#include "ariaTypedefs.h"
#include "ArFunctor.h"
#include "ArTaskPoolTask.h"
#include "ArPeriodicTask.h"
#include "ArMutex.h"
#include "ArSemaphore.h"


/// A thread pool for running functions or ArPeriodicTasks
class ArTaskPool
{
public:

  /// Get a pointer to the 'single' instance of ArTaskPool
  AREXPORT static ArTaskPool * getPool();
  /// Set the pointer to the 'single' instance of ArTaskPool
  AREXPORT static void setPool(ArTaskPool *pool);

  /// Constructor
  AREXPORT ArTaskPool();

  /// Destructor
  AREXPORT virtual ~ArTaskPool();

  /// Run a function as a periodic task
  AREXPORT virtual bool runTask(ArFunctor *task);
  /// Run a periodic task
  AREXPORT virtual bool runTask(ArPeriodicTask *task);

  /// Init the pool.
  AREXPORT virtual void init(int minLimit=1, int maxLimit=0);

  /// Uninit the pool. Blocks till all tasks exit.
  AREXPORT virtual void uninit();

  /// Force all tasks to exit, then uninit the pool.
  AREXPORT virtual void forceUninit();

  AREXPORT virtual bool runTask(std::pair<ArFunctor*,ArPeriodicTask*> task);
  AREXPORT virtual void makeThread(bool makeBusy);
  AREXPORT virtual void reset();

  static ArTaskPool *ourPool;

  bool myRunning;
  int myMaxLimit;
  int myMinLimit;
  std::set<ArTaskPoolTask*> myThreads;
  std::set<ArTaskPoolTask*> myFreeThreads;
  std::set<ArTaskPoolTask*> myBusyThreads;
  std::list<std::pair<ArFunctor*,ArPeriodicTask*> > myTaskQueue;
  ArMutex myMutex;
  ArSemaphore myThreadIdlerSem;
};


#endif // _ARTASKPOOL_H
