/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArResolver.h"
#include "ArAction.h"
#include "ArLog.h"
#include "ArRobot.h"

AREXPORT ArAction::ArAction(const char *name, const char *description)
{
  myRobot = NULL;
  myNumArgs = 0;
  myName = name;
  myDescription = description;
  myIsActive = true;
}

AREXPORT ArAction::~ArAction(void)
{
  if (myRobot != NULL)
    myRobot->remAction(this);
}

AREXPORT std::string ArAction::getName(void)
{
  return myName;
}

AREXPORT std::string ArAction::getDescription(void)
{
  return myDescription;
}

AREXPORT int ArAction::getNumArgs(void)
{
  return myNumArgs;
}

AREXPORT void ArAction::setNextArgument(ArArg const &arg)
{
  myArgumentMap[myNumArgs] = arg;
  myNumArgs++;
}

AREXPORT ArArg *ArAction::getArg(int number)
{
  std::map<int, ArArg>::iterator it;
  
  it = myArgumentMap.find(number);
  if (it != myArgumentMap.end())
    return &(*it).second;
  else
    return NULL;
}

AREXPORT void ArAction::setRobot(ArRobot *robot)
{
  myRobot = robot;
}

AREXPORT bool ArAction::isActive(void)
{
  return myIsActive;
}

AREXPORT void ArAction::activate(void)
{
  myIsActive = true;
}

AREXPORT void ArAction::deactivate(void)
{
  myIsActive = false;
}

AREXPORT void ArAction::print(void)
{
  int i;
  std::string str;
  ArArg *arg;

  ArLog::log(ArLog::Terse, "isActive %d", myIsActive);
  if (getDescription().size() != 0)
    ArLog::log(ArLog::Terse, "Action %s is described as: %s", 
	       getName().c_str(), getDescription().c_str());
  else
    ArLog::log(ArLog::Terse, "Action %s has no description.", 
	       getName().c_str());
  if (getNumArgs() == 0)
    ArLog::log(ArLog::Terse, "Action %s has no arguments.\n", 
	       getName().c_str());
  else
  {
    ArLog::log(ArLog::Terse, "Action %s has %d arguments, of type(s):", 
	       (getName().c_str()), getNumArgs());

    for (i = 0; i < getNumArgs(); i++) 
    {
      arg = getArg(i);
      if (arg == NULL)
	continue;
      arg->print();
    }
    ArLog::log(ArLog::Terse, "");
  }
}



