/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArActionGroup.h"
#include "ArActionGroups.h"
#include "ArActionLimiterTableSensor.h"
#include "ArActionLimiterForwards.h"
#include "ArActionLimiterBackwards.h"
#include "ArActionInput.h"
#include "ArActionStop.h"
#include "ArActionStallRecover.h"
#include "ArActionBumpers.h"
#include "ArActionAvoidFront.h"
#include "ArActionConstantVelocity.h"
#include "ArActionJoydrive.h"
#include "ArActionKeydrive.h"



AREXPORT ArActionGroupInput::ArActionGroupInput(ArRobot *robot)
  : ArActionGroup(robot)
{
  addAction(new ArActionLimiterTableSensor, 100);
  addAction(new ArActionLimiterForwards("Speed Limiter Near", 
                                                      300, 600, 250),
                          90);
  addAction(new ArActionLimiterForwards("Speed Limiter Far",
                                                      300, 1100, 400),
                          89);
  addAction(new ArActionLimiterBackwards, 80);
  myInput = new ArActionInput;
  addAction(myInput, 70);
}

AREXPORT ArActionGroupInput::~ArActionGroupInput(void)
{
  std::list<ArAction *>::iterator it;
  removeActions();
  for (it = myActions.begin(); it != myActions.end(); ++it)
  {
    delete (*it);
  }
}

AREXPORT void ArActionGroupInput::setVel(double vel)
{
  myInput->setVel(vel);
}

AREXPORT void ArActionGroupInput::deltaVel(double delta)
{
  myInput->deltaVel(delta);
}

AREXPORT void ArActionGroupInput::deltaHeading(double delta)
{
  myInput->deltaHeading(delta);
}

AREXPORT ArActionGroupStop::ArActionGroupStop(ArRobot *robot)
  : ArActionGroup(robot)
{
  addAction(new ArActionStop, 100);
}

AREXPORT ArActionGroupStop::~ArActionGroupStop(void)
{
  std::list<ArAction *>::iterator it;
  removeActions();
  for (it = myActions.begin(); it != myActions.end(); ++it)
  {
    delete (*it);
  }
}


AREXPORT ArActionGroupTeleop::ArActionGroupTeleop(ArRobot *robot)
  : ArActionGroup(robot)
{
  ArActionJoydrive *joydrive;
  addAction(new ArActionLimiterTableSensor, 100);
  addAction(new ArActionLimiterForwards("Speed Limiter Near", 
                                                      300, 600, 250),
                          90);
  addAction(new ArActionLimiterForwards("Speed Limiter Far",
                                                      300, 1100, 400),
                          89);
  addAction(new ArActionLimiterBackwards, 80);
  joydrive = new ArActionJoydrive;
  joydrive->setStopIfNoButtonPressed(false);
  addAction(joydrive, 70);
  addAction(new ArActionKeydrive, 69);
}

AREXPORT ArActionGroupTeleop::~ArActionGroupTeleop(void)
{
  std::list<ArAction *>::iterator it;
  removeActions();
  for (it = myActions.begin(); it != myActions.end(); ++it)
  {
    delete (*it);
  }
}

AREXPORT ArActionGroupWander::ArActionGroupWander(ArRobot *robot)
  : ArActionGroup(robot)
{
  addAction(new ArActionStallRecover, 100);
  addAction(new ArActionBumpers, 90);
  addAction(new ArActionAvoidFront("Avoid Front Near", 225, 0), 
            80);
  addAction(new ArActionAvoidFront, 79);
  addAction(new ArActionConstantVelocity("Constant Velocity",
                                         400),
            50);

}

AREXPORT ArActionGroupWander::~ArActionGroupWander(void)
{
  std::list<ArAction *>::iterator it;
  removeActions();
  for (it = myActions.begin(); it != myActions.end(); ++it)
  {
    delete (*it);
  }
}
