/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArActionInput.h"
#include "ArRobot.h"
/**
   @param name name of the action
*/
AREXPORT ArActionInput::ArActionInput(const char *name) :
    ArAction(name, "Inputs vel and heading")
{
  myUsingVel = 0;
  myVelSet = 0;
  myDeltaHeading = 0;
  myHeadingDesired = 0;
}

AREXPORT ArActionInput::~ArActionInput(void)
{
}

AREXPORT void ArActionInput::setVel(double vel)
{
  myUsingVel = true;
  myVelSet = vel;
}

AREXPORT void ArActionInput::deltaVel(double delta)
{
  myUsingVel = false;
  myDeltaVel += delta;
}

AREXPORT void ArActionInput::deltaHeading(double delta)
{
  myDeltaHeading += delta;
}

AREXPORT void ArActionInput::activate(void)
{
  if (!isActive())
  {
    myUsingVel = true;
    myVelSet = 0;
    myDeltaHeading = 0;
  }
  myIsActive = true;
}

AREXPORT ArActionDesired *ArActionInput::fire(
	ArActionDesired currentDesired)
{
  myDesired.reset();

  if (!myUsingVel)
  {
    /* if (myVelDesired > 0 && myVelDesired > myRobot->getVel())
      myVelDesired = myRobot->getVel();
    if (myVelDesired < 0 && myVelDesired < myRobot->getVel())
    myVelDesired = myRobot->getVel(); */
    myVelDesired += myDeltaVel;
    myDesired.setVel(myVelDesired);
    myDeltaVel = 0;
  }
  else
    myDesired.setVel(myVelSet);
  myHeadingDesired += myDeltaHeading;
  myDesired.setHeading(ArMath::fixAngle(myHeadingDesired));
  myDeltaHeading = 0;

  return &myDesired;
}
