/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArActionLimiterBackwards.h"
#include "ArRobot.h"

/**
   @param name name of the action
   @param stopDistance distance at which to stop (mm)
   @param slowDistance distance at which to slow down (mm)
   @param maxBackwardsSpeed maximum backwards speed, speed allowed scales
   from this to 0 at the stop distance (mm/sec)
*/
AREXPORT ArActionLimiterBackwards::ArActionLimiterBackwards(const char *name, 
						    double stopDistance,
						    double slowDistance,
						    double maxBackwardsSpeed) :
  ArAction(name,
	   "Slows the robot down so as not to hit anything in front of it.")
{
  setNextArgument(ArArg("stop distance", &myStopDist, 
			"Distance at which to stop. (mm)"));
  myStopDist = stopDistance;

  setNextArgument(ArArg("slow distance", &mySlowDist, 
			"Distance at which to slow down. (mm)"));
  mySlowDist = slowDistance;

  setNextArgument(ArArg("maximum backwards speed", &myMaxBackwardsSpeed, 
			 "Maximum backwards speed, scales from this to 0 at stopDistance (-mm/sec)"));
  myMaxBackwardsSpeed = maxBackwardsSpeed;
  
}

AREXPORT ArActionLimiterBackwards::~ArActionLimiterBackwards(void)
{

}

AREXPORT ArActionDesired *
ArActionLimiterBackwards::fire(ArActionDesired currentDesired)
{
  double dist;
  double maxVel;

  myDesired.reset();
  dist = myRobot->checkRangeDevicesCurrentBox(-myRobot->getRobotRadius()/2,
					      -myRobot->getRobotRadius() * 1.5,
			       mySlowDist + (-myRobot->getRobotRadius()),
					      myRobot->getRobotRadius() * 1.5);
  dist -= myRobot->getRobotRadius();
  if (dist > -mySlowDist)
  {
    //printf("backwards nothing\n");
    myDesired.setMaxNegVel(myMaxBackwardsSpeed);
    return &myDesired;
  }
      
  if (dist < -myStopDist)
  {
    //printf("backwards stop\n");
    myDesired.setMaxNegVel(0);
    return &myDesired;
  }
			
  maxVel = myMaxBackwardsSpeed * ((-dist - myStopDist) / (mySlowDist - myStopDist));
  //printf("Neg Max vel %f (stopdist %.1f slowdist %.1f slowspeed %.1f\n", maxVel,	 myStopDist, mySlowDist, myMaxBackwardsSpeed);
  myDesired.setMaxNegVel(maxVel);
  return &myDesired;
  
}
