/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArJoyHandler.h"

bool ArJoyHandler::init(void)
{
  if ((myJoyDesc = fopen("/dev/js0", "r")) != NULL) {
    myPhysMax = 255;
    myInitialized = true;
    startCal();
    endCal();
    return true;
  } else {
    myInitialized = false;
    return false;
  }
}

/**
   This returns the raw value from the joystick... with X and Y 
   varying between -128 and poseitive 128... this shouldn't be used 
   except in calibration since it'll give very strange readings.  
   For example its not uncommon for a joystick to move 10 to the right
   but 50 or 100 to the left, so if you aren't adjusting for this you get
   a robot (or whatever) that goes left really fast, but will hardly go right,
   hence you should use getAdjusted exclusively except for display in 
   calibration.
   @param x pointer to an integer in which to store x value
   @param y pointer to an integer in which to store y value
*/
void ArJoyHandler::getUnfiltered(int *x, int *y)
{
  if (myJoyDesc == NULL || !myInitialized || 
      fread(&myJoyData, 1, JS_RETURN, myJoyDesc) != JS_RETURN) 
  {
    *x = 0;
    *y = 0;
  } 
  else 
  {
    *x = myJoyData.x - 128; 
    *y =  - (myJoyData.y - 128);
    if (*x > myMaxX)
      myMaxX = *x;
    if (*x < myMinX)
      myMinX = *x;
    if (*y > myMaxY)
      myMaxY = *y;
    if (*y < myMinY)
      myMinY = *y;
  } 
}

/**
   @param button button to test for pressed, buttons are 1 through 4
   @return true if the button is pressed, false otherwise
*/
bool ArJoyHandler::getButton(int button)
{
  if (myJoyDesc == NULL || !myInitialized || 
      fread(&myJoyData, 1, JS_RETURN, myJoyDesc) != JS_RETURN) 
    return false;
  else if (button == 1)
    return (bool) (myJoyData.buttons & 1);
  else if (button == 2)
    return (bool) (myJoyData.buttons & 2);
  else if (button == 3)
    return (bool) (myJoyData.buttons & 4);
  else if (button == 4)
    return (bool) (myJoyData.buttons & 8);
  else {
    //printf("Bad button passed to ArJoyHandler::getButton\n");
    return false;
  }

}

