/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArJoyHandler.h"

AREXPORT bool ArJoyHandler::init(void)
{
  if (joyGetDevCaps(JOYSTICKID1,&myJoyCaps,sizeof(myJoyCaps)) == 
      JOYERR_NOERROR &&
      joyGetPos(JOYSTICKID1,&myJoyInfo) != JOYERR_UNPLUGGED) {
    myPhysMax = myJoyCaps.wXmax - myJoyCaps.wXmin;
    myInitialized = true;
    startCal();
    endCal();
    return true;
  } else {
    myInitialized = false;
    return false;
  }
}

AREXPORT void ArJoyHandler::getUnfiltered(int *x, int *y)
{
  if (joyGetPos(JOYSTICKID1,&myJoyInfo) == JOYERR_UNPLUGGED) {
    *x = 0;
    *y = 0;
  } else {
    *x = (int)(myJoyInfo.wXpos*256.0/myPhysMax)-128;
    *y = (int)-((myJoyInfo.wYpos*256.0/myPhysMax)-128);
    if (*x > myMaxX)
      myMaxX = *x;
    if (*x < myMinX)
      myMinX = *x;
    if (*y > myMaxY)
      myMaxY = *y;
    if (*y < myMinY)
      myMinY = *y;
  }
}

bool ArJoyHandler::getButton(int button)
{
  if (joyGetPos(JOYSTICKID1,&myJoyInfo) == JOYERR_UNPLUGGED) 
    return false;
  else if (button == 1)
    return (bool) (myJoyInfo.wButtons & 1);
  else if (button == 2)
    return (bool) (myJoyInfo.wButtons & 2);
  else if (button == 3)
    return (bool) (myJoyInfo.wButtons & 4);
  else if (button == 4)
    return (bool) (myJoyInfo.wButtons & 8);
  else {
    //printf("Bad button passed to ArJoyHandler::getButton\n");
    return false;
  }
}


