/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArLog.h"

#include <stdarg.h>


ArMutex ArLog::ourMutex;
ArLog::LogType ArLog::ourType=StdOut;
ArLog::LogLevel ArLog::ourLevel=ArLog::Normal;
FILE * ArLog::ourFP=0;
std::string ArLog::ourFileName;
int ArLog::ourColbertStream = -1;
AREXPORT void (* ArLog::colbertPrint)(int i, char *str);

/**
   This function is used like printf(). If the supplied level is less than
   or equal to the set level, it will be printed.
   @param level level of logging
   @param str printf() like formating string
*/
AREXPORT void ArLog::log(LogLevel level, char *str, ...)
{
  char buf[2048];

  ourMutex.lock();
  if (level <= ourLevel)
  {
    va_list ptr;
    va_start(ptr, str);
    vsprintf(buf, str, ptr);
    // can do whatever you want with the buf now
    if (ourType == Colbert)
    {
      if (colbertPrint)		// check if we have a print routine
	(*colbertPrint)(ourColbertStream, buf);
    }
    else if (ourFP)
      fprintf(ourFP, "%s\n", buf);
    else if (ourType != None)
      printf("%s\n", buf);
    va_end(ptr);
  }
  ourMutex.unlock();
}

/**
   Initialize the logging utility by supplying the type of logging and the
   level of logging. If the type is File, the fileName needs to be supplied.
   @param type type of Logging
   @param level level of logging
   @param fileName the name of the file for File type of logging
*/
AREXPORT bool ArLog::init(LogType type, LogLevel level, const char *fileName)
{
  close();
  ourType=type;
  ourLevel=level;
  if (type == StdOut)
    ourFP=stdout;
  else if (type == StdErr)
    ourFP=stderr;
  else if (type == File)
  {
    if (fileName)
    {
      ourFP=fopen(fileName, "w");
      if (ourFP)
	return(false);
      ourFileName=fileName;
    }
  }
  else if (type == Colbert)
  {
    colbertPrint = NULL;
    ourColbertStream = -1;	// use default stream
    if (fileName)
    {  // need to translate fileName to integer index
    }
  }

  return(true);
}

AREXPORT void ArLog::close()
{
  if (ourFP && (ourType == File))
  {
    fclose(ourFP);
    ourFP=0;
    ourFileName="";
  }
}

AREXPORT void ArLog::logNoLock(LogLevel level, char *str, ...)
{
  char buf[1024];

  if (level <= ourLevel)
  {
    va_list ptr;
    va_start(ptr, str);
    vsprintf(buf, str, ptr);
    // can do whatever you want with the buf now
    if (ourType == Colbert)
    {
      if (colbertPrint)		// check if we have a print routine
	(*colbertPrint)(ourColbertStream, buf);
    }
    else if (ourFP)
      fprintf(ourFP, "%s\n", buf);
    else if (ourType != None)
      printf("%s\n", buf);
    va_end(ptr);
  }
}
