/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArPrefSection.h"
#include <stdio.h>

AREXPORT ArPrefSection::ArPrefSection(int id, const char *name,
				      const char *comment) :
  myID(id),
  myName(name),
  myComment(comment),
  myPrefs(0),
  myLastPref(0),
  myNext(0),
  myPrev(0)
{
}

AREXPORT ArPrefSection::ArPrefSection(const ArPrefSection &sect) :
  myID(sect.myID),
  myName(sect.myName),
  myComment(sect.myComment),
  myPrefs(0),
  myLastPref(0),
  myNext(0),
  myPrev(0)
{
}

AREXPORT ArPrefSection::~ArPrefSection()
{
}

ArPrefSection * ArPrefSection::findSect(int id)
{
  ArPrefSection *psect;

  for (psect=this; psect && (psect->myID != id); psect=psect->myNext)
    ;

  return(psect);
}

ArPrefSection * ArPrefSection::findSect(const char *id)
{
  ArPrefSection *psect;

  for (psect=this; psect && (psect->myName.compare(id) != 0);
       psect=psect->myNext)
    ;

  return(psect);
}

ArPref * ArPrefSection::findPref(int id)
{
  ArPref *pref;

  for (pref=myPrefs; pref && (pref->myPref != id); pref=pref->myNext)
    ;

  return(pref);
}

ArPref * ArPrefSection::findPref(const char *id)
{
  ArPref *pref;

  for (pref=myPrefs; pref && (pref->myName.compare(id) != 0);
       pref=pref->myNext)
    ;

  return(pref);
}

void ArPrefSection::addPref(ArPref *pref)
{
  ArPref *ppref;

  pref->myNext=pref->myPrev=0;

  // Do we have a pref of this type already?
  for (ppref=myPrefs; ppref && (ppref->myPref != pref->myPref);
       ppref=ppref->myNext)
    ;
  // Lets find the last pref of this type
  for (; ppref && ppref->myNext && (ppref->myNext->myPref == pref->myPref);
       ppref=ppref->myNext)
    ;
  // Okay, lets now insert the pref. If no existing pref was found, just tack
  // in on the end
  if (ppref)
  {
    pref->myNext=ppref->myNext;
    pref->myPrev=ppref;
    ppref->myNext=pref;
    if (pref->myNext)
      pref->myNext->myPrev=pref;
  }
  else if (myLastPref)
  {
    myLastPref->myNext=pref;
    pref->myNext=0;
    pref->myPrev=myLastPref;
    myLastPref=pref;
  }
  else
  {
    myPrefs=pref;
    myLastPref=pref;
    pref->myNext=0;
    pref->myPrev=0;
  }
}

void ArPrefSection::delPref(ArPref *pref)
{
  if (pref->myNext)
    pref->myNext->myPrev=pref->myPrev;
  if (pref->myPrev)
    pref->myPrev->myNext=pref->myNext;
  if (myPrefs == pref)
    myPrefs=pref->myNext;
  if (myLastPref == pref)
    myLastPref=pref->myPrev;
  delete pref;
}

