/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"

#include "ariaOSDef.h"
#include "ArRobotParams.h"

AREXPORT ArRobotParams::ArRobotParams(void)
{
  myClass = "";
  mySubClass = "";
  myRobotRadius = 250;
  myRobotDiagonal = 120;
  myHolonomic = true;
  myMaxVelocity = 0;
  myMaxRVelocity = 0;
  myAngleConvFactor = 0;
  myDistConvFactor = 0;
  myRangeConvFactor = 0;
  myNumSonar = 0;
}

AREXPORT ArRobotParams::~ArRobotParams(void)
{

}

AREXPORT void ArRobotParams::init(ArRobotParamFile *param)
{
  ArPref *pref, *pfirst, *plast;
  int *intSet=0, numInts;
  int num = -1, x = 0, y = 0, th = 0;

  if (param == NULL)
    return;

  if ((pref = param->getPref(ArRobotParamFile::General, 
				 ArRobotParamFile::Class)) != NULL)
    myClass = pref->getString();
  if ((pref = param->getPref(ArRobotParamFile::General, 
				 ArRobotParamFile::Subclass)) != NULL)
    mySubClass = pref->getString();
  if ((pref = param->getPref(ArRobotParamFile::General, 
				 ArRobotParamFile::RobotRadius)) != NULL)
    myRobotRadius = pref->getDouble();
  if ((pref = param->getPref(ArRobotParamFile::General, 
				 ArRobotParamFile::RobotDiagonal)) != NULL)
    myRobotDiagonal = pref->getDouble();
  if ((pref = param->getPref(ArRobotParamFile::General, 
				 ArRobotParamFile::Holonomic)) != NULL)
  {
    if (pref->getInt() != 0)
      myHolonomic = true;
    else
      myHolonomic = false;
  }
  if ((pref = param->getPref(ArRobotParamFile::General, 
				 ArRobotParamFile::HasMoveCommand)) != NULL)
  {
    if (pref->getInt() != 0)
      myHaveMoveCommand = true;
    else
      myHaveMoveCommand = false;
  }
  if ((pref = param->getPref(ArRobotParamFile::General, 
				 ArRobotParamFile::MaxVelocity)) != NULL)
    myMaxVelocity = pref->getInt();
  if ((pref = param->getPref(ArRobotParamFile::General, 
				 ArRobotParamFile::MaxRVelocity)) != NULL)
    myMaxRVelocity = pref->getInt();
  if ((pref = param->getPref(ArRobotParamFile::General,
                                 ArRobotParamFile::RequestIOPackets)) != NULL)
    myRequestIOPackets = pref->getBool();

  if ((pref = param->getPref(ArRobotParamFile::ConvFactors, 
				 ArRobotParamFile::AngleConvFactor)) != NULL)
    myAngleConvFactor = pref->getDouble();
  if ((pref = param->getPref(ArRobotParamFile::ConvFactors, 
				 ArRobotParamFile::DistConvFactor)) != NULL)
    myDistConvFactor = pref->getDouble();
  if ((pref = param->getPref(ArRobotParamFile::ConvFactors, 
				 ArRobotParamFile::VelConvFactor)) != NULL)
    myVelConvFactor = pref->getDouble();
  if ((pref = param->getPref(ArRobotParamFile::ConvFactors, 
				 ArRobotParamFile::RangeConvFactor)) != NULL)
    myRangeConvFactor = pref->getDouble();
  if ((pref = param->getPref(ArRobotParamFile::ConvFactors, 
				 ArRobotParamFile::DiffConvFactor)) != NULL)
    myDiffConvFactor = pref->getDouble();
  if ((pref = param->getPref(ArRobotParamFile::ConvFactors, 
				 ArRobotParamFile::Vel2Divisor)) != NULL)
    myVel2Divisor = pref->getDouble();

  if ((pref = param->getPref(ArRobotParamFile::Accessories, 
			     ArRobotParamFile::TableSensingIR)) != NULL)
    myTableSensingIR = pref->getBool();
  if ((pref = param->getPref(ArRobotParamFile::Accessories,
                             ArRobotParamFile::NewTableSensingIR)) != NULL)
    myNewTableSensingIR = pref->getBool();
  if ((pref = param->getPref(ArRobotParamFile::Accessories, 
			     ArRobotParamFile::FrontBumpers)) != NULL)
    myFrontBumpers = pref->getBool();
  if ((pref = param->getPref(ArRobotParamFile::Accessories,
                             ArRobotParamFile::NumFrontBumpers)) != NULL)
    myNumFrontBumpers = pref->getInt();
  if ((pref = param->getPref(ArRobotParamFile::Accessories, 
			     ArRobotParamFile::RearBumpers)) != NULL)
    myRearBumpers = pref->getBool();
  if ((pref = param->getPref(ArRobotParamFile::Accessories,
                             ArRobotParamFile::NumRearBumpers)) != NULL)
    myNumRearBumpers = pref->getInt();

  if ((pref = param->getPref(ArRobotParamFile::Sonar, 
			     ArRobotParamFile::SonarNum)) != NULL)
    myNumSonar = pref->getInt();
  if (param->getPrefRange(ArRobotParamFile::Sonar,
			  ArRobotParamFile::SonarUnit, &pfirst, &plast))
  {
    for (num = -1, x = 0, y = 0, th = 0, pref=pfirst; pref; )
    {
      numInts=pref->getSetCount();
      if (intSet)
	delete[] intSet;
      intSet=new int[numInts];
      if (pref->getIntSet(intSet, numInts))
      {
	/*
	  printf("sonar ");
	  for (intIter=intSet.begin(); intIter != intSet.end(); ++intIter)
	  printf("%d ", (*intIter));
	  printf("\n");
	*/
	num=intSet[0];
	mySonarMap[num];

	x = intSet[1];
	mySonarMap[num][SONAR_X] = x;
	y = intSet[2];
	mySonarMap[num][SONAR_Y] = y;
	th = intSet[3];
	mySonarMap[num][SONAR_TH] = th;

	if (pref == plast)
	  break;
	else
	  pref=pref->myNext;
      }
    }
  }
  if ((pref = param->getPref(ArRobotParamFile::Laser, 
			     ArRobotParamFile::LaserPossessed)) != NULL)
    myLaserPossessed = pref->getBool();
  if ((pref = param->getPref(ArRobotParamFile::Laser, 
				 ArRobotParamFile::LaserPort)) != NULL)
    myLaserPort = pref->getString();
  if ((pref = param->getPref(ArRobotParamFile::Laser, 
			     ArRobotParamFile::LaserPowerControlled)) != NULL)
    myLaserPowerControlled = pref->getBool();
  if ((pref = param->getPref(ArRobotParamFile::Laser, 
			     ArRobotParamFile::LaserFlipped)) != NULL)
    myLaserFlipped = pref->getBool();
  if ((pref = param->getPref(ArRobotParamFile::Laser, 
				 ArRobotParamFile::LaserX)) != NULL)
    myLaserX = pref->getInt();
  if ((pref = param->getPref(ArRobotParamFile::Laser, 
				 ArRobotParamFile::LaserY)) != NULL)
    myLaserY = pref->getInt();
}
