/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArTaskPoolTask.h"
#include "ArTaskPool.h"
#include "ArLog.h"


AREXPORT ArTaskPoolTask::ArTaskPoolTask(ArTaskPool *pool) :
  myPool(pool),
  myIdlerFunc(this, &ArTaskPoolTask::idler)
{
}

AREXPORT ArTaskPoolTask::~ArTaskPoolTask()
{
}

AREXPORT int ArTaskPoolTask::create()
{
  return(ArThread::create(&myIdlerFunc));
}

AREXPORT void ArTaskPoolTask::idler()
{
  std::pair<ArFunctor*, ArPeriodicTask*> task;
  std::list<std::pair<ArFunctor*,ArPeriodicTask*> >::iterator iter;

  while (myRunning)
  {
    task.first=0;
    task.second=0;

    // Wait for a task
    myPool->myThreadIdlerSem.wait();

    // Check to see if we were woken up to be killed.
    if (!myRunning)
      break;

    // Obtain the lock and the task, then clear the task
    myPool->myMutex.lock();
    iter=myPool->myTaskQueue.begin();
    if (iter != myPool->myTaskQueue.end())
    {
      task=(*iter);
      myPool->myTaskQueue.pop_front();
      // Mark ourselves as busy
      myPool->myFreeThreads.erase(this);
      myPool->myBusyThreads.insert(this);
    }
    myPool->myMutex.unlock();

    // Lets do it
    if (task.first)
      task.first->invoke();
    else if (task.second)
    {
      task.second->runTask();
      task.second->setRunning(false);
    }

    // Directly bail if we are already killed
    if (!myRunning)
      break;

    // Check to see if we need to go die
    myPool->myMutex.lock();
    if (!myPool->myMinLimit || 
	(myPool->myMaxLimit && 
	 (((signed int)myPool->myThreads.size()) > myPool->myMaxLimit)))
    {
      myPool->myThreads.erase(this);
      myRunning=false;
    }
    myPool->myMutex.unlock();
  }
}
