/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include <list>
#ifdef linux
#include <unistd.h>
#endif
#ifdef WIN32
#include <time.h>
#endif
#include "ArTimer.h"


ArTimer * ArTimer::ourInst=NULL;


AREXPORT ArTimer::ArTimer() :
  myTimers()
{
}

AREXPORT ArTimer::~ArTimer()
{
}

AREXPORT void ArTimer::addAt(time_t time, ArFunctor *func)
{
  ourInst->lock();
  ourInst->myTimers.insert(MapType::value_type(time, func));
  ourInst->unlock();
}

AREXPORT void ArTimer::addDelta(time_t delta, ArFunctor *func)
{
  ourInst->lock();
  ourInst->myTimers.insert(MapType::value_type(time(0)+delta, func));
  ourInst->unlock();
}

AREXPORT bool ArTimer::init()
{
  ourInst->ourInst=new ArTimer;
  return(ourInst->create());
}

AREXPORT void * ArTimer::runThread(void *)
{
  MapType::iterator start, end;

  while (myRunning)
  {
    if ((*myTimers.begin()).first < time(0))
#ifdef WIN32
      Sleep(((*myTimers.begin()).first-time(0)) * 1000);
#else
      sleep((*myTimers.begin()).first-time(0));
#endif
    lock();
    for (start=end=myTimers.begin();
	 ((*end).first <= time(0)) && (end != myTimers.end()); ++end)
      (*end).second->invoke();
    if ((start != myTimers.end()) && (end != myTimers.end()))
    {
      if ((*end).first > time(0))
	--end;
      myTimers.erase(start, end);
    }
    unlock();
  }

  return(NULL);
}
