/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "Aria.h"


void printPrefSet(ArPreferences &prefs, ArPref::ValType type, int section,
		  int pref)
{
  int i, size;
  ArPref *p;
  bool boolSet[100];
  int intSet[100];
  double doubleSet[100];
  std::string stringSet[100];

  printf("set=");
  if (type == ArPref::Boolean)
  {
    p=prefs.getPref(section, pref);
    size=p->getBoolSet(boolSet, sizeof(boolSet));
    for (i=0; i<size; ++i)
    {
      if (boolSet[i])
	printf("true ");
      else
	printf("false ");
    }
  }
  else if (type == ArPref::Integer)
  {
    p=prefs.getPref(section, pref);
    size=p->getIntSet(intSet, sizeof(intSet));
    for (i=0; i<size; ++i)
      printf("%d ", intSet[i]);
  }
  else if (type == ArPref::Double)
  {
    p=prefs.getPref(section, pref);
    size=p->getDoubleSet(doubleSet, sizeof(doubleSet));
    for (i=0; i<size; ++i)
      printf("%e ", doubleSet[i]);
  }
  else if (type == ArPref::String)
  {
    p=prefs.getPref(section, pref);
    size=p->getStringSet(stringSet, sizeof(stringSet));
    for (i=0; i<size; ++i)
      printf("%s ", stringSet[i].c_str());
  }

  printf("\n");
}

int main()
{
  int i, size, intSet[100];
  std::string val;
  ArRobotAmigo amigo;
  ArPref pref(ArRobotAmigo::General, ArRobotAmigo::LASTPREF,
	      "test", "true false true true false false", 0,
	      ArPref::Boolean, "This is a comment");
  ArPref *p, *pfirst, *plast;

  amigo.addPref(&pref);
  amigo.init();
  amigo.load();

  p=amigo.getPref(ArRobotAmigo::Sonar, ArRobotAmigo::SonarNum);
  printf("SonarNum %d\n", p->getInt());

  amigo.getPrefRange(ArRobotAmigo::Sonar, ArRobotAmigo::SonarUnit,
		     &pfirst, &plast);
  for (; pfirst;)
  {
    p=pfirst;
    size=p->getIntSet(intSet, sizeof(intSet));
    printf("SonarUnit ");
    for (i=0; i<size; ++i)
      printf("%d ", intSet[i]);
    printf("\n");
    if (pfirst == plast)
      pfirst=0;
    else
      pfirst=pfirst->myNext;
  }

  p=amigo.getPref(ArRobotAmigo::General, ArRobotAmigo::MaxVelocity);
  printf("vel=%d\n", p->getInt());

  printf("Default value: ");
  printPrefSet(amigo, ArPref::Boolean, ArRobotAmigo::General,
	       ArRobotAmigo::LASTPREF);
  printf("Setting value to 'false false true false true true'\n");
  p=amigo.getPref(ArRobotAmigo::General, ArRobotAmigo::LASTPREF);
  p->setBoolSet(false, 6, false, false, true, false, true, true);
  printPrefSet(amigo, ArPref::Boolean, ArRobotAmigo::General,
  ArRobotAmigo::LASTPREF);
  //amigo.save();

  return(0);
}
