/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "Aria.h"

ArRobot *robot;

ArSonarDevice sonar;

void sonarPrinter(void)
{
  ArSonarDevice *sd;
  
  std::list<ArPose *>::iterator it;
  std::list<ArPose *> *readings;
  ArPose *pose;

  sd = (ArSonarDevice *)robot->findRangeDevice("sonar");
  if (sd != NULL)
  {
    sd->lockDevice();
    readings = sd->getCurrentBuffer();
    if (readings != NULL)
    {
      for (it = readings->begin(); it != readings->end(); ++it)
      {
	pose = (*it);
	//pose->print();
      }
    }
    sd->unlockDevice();
  }
  /*

  double d;
  double th;
  int i;
	
  printf("\r");

  d = sonar.currentReadingPolar(-70, 70, &th);
  printf(" front %5.0f  ", d);
  if (d != 5000)
    printf("%3.0f ", th);

  d = sonar.currentReadingPolar(-100, 0, &th);
  printf(" right %5.0f ", d);
  if (d != 5000)
    printf("%3.0f ", th);

  d = sonar.currentReadingPolar(0, 100, &th);
  printf(" left %5.0f ", d);
  if (d != 5000)
    printf("%3.0f ", th);

  fflush(stdout);
  */
}

int main(void)
{
  std::string str;
  int ret;
  ArGlobalFunctor sonarPrinterCB(&sonarPrinter);
  ArTcpConnection con;
  Aria::init();

  robot = new ArRobot;

  if ((ret = con.open()) != 0)
  {
    str = con.getOpenMessage(ret);
    printf("Open failed: %s\n", str.c_str());
    exit(0);
  }

  robot->setDeviceConnection(&con);
  if (!robot->blockingConnect())
  {
    printf("Could not connect to robot->.. exiting\n");
    exit(0);
  }

  robot->addRangeDevice(&sonar);

  robot->addUserTask("Sonar printer", 50, &sonarPrinterCB);
  robot->comInt(ArCommands::SONAR, 1);
  robot->comInt(ArCommands::SOUNDTOG, 0);

  robot->run(true);
  Aria::shutdown();


}
