/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "Aria.h"
//#include <time.h>

class TestThread : public ArASyncTask
{
public:
  TestThread(int number, ArMutex &mutex);
  ~TestThread(void) {}

  virtual void * runThread(void *arg);

  int myNum;
  ArMutex &myMutex;
};

TestThread::TestThread(int number, ArMutex &mutex) :
  myNum(number),
  myMutex(mutex)
{
}

void *TestThread::runThread(void *arg)
{
  unsigned int interval;

  while (myRunning)
  {
    // Yield the processor here so that the other threads get a chance
    // to get the lock. calling ArThread::yield() will give sequential
    // running of the threads.
    //yield();

    // Sleeping of 1 microsecond randomizes the order of running of the
    // threads.
    ArUtil::sleep(1);

    myMutex.lock();
    interval=rand()%3000;
    printf("Thread %d: Locked. Going to sleep for %dms\n", myNum, interval);
    ArUtil::sleep(interval);
    printf("Thread %d: Unlocking...\n", myNum);
    myMutex.unlock();
  }

  return NULL;
}

int main(int argc, char **argv) 
{
  ArMutex mutex;
  TestThread thread1(1, mutex), thread2(2, mutex), thread3(3, mutex),
    thread4(4, mutex);

  srand(time(0));

  Aria::init(Aria::SIGHANDLE_THREAD, false);

  thread1.create();
  thread2.create();
  thread3.create();
  thread4.runInThisThread();

  Aria::shutdown();

  return(0);
}

