/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARACTIONBUMPERS_H
#define ARACTIONBUMPERS_H

#include "ariaTypedefs.h"
#include "ArAction.h"

/// Action to deal with if the bumpers trigger
/**
   This class basically responds to the bumpers the robot has, what
   the activity things the robot has is decided by the param file.  If
   the robot is going forwards and bumps into something with the front
   bumpers, it will back up and turn.  If the robot is going backwards
   and bumps into something with the rear bumpers then the robot will
   move forward and turn.  
*/

class ArActionBumpers : public ArAction
{
public:
  /// Constructor
  AREXPORT ArActionBumpers(const char *name = "bumpers", 
			   double backOffSpeed = 100, int backOffTime = 2500,
			   int turnTime = 500, bool setMaximums = false);
  /// Destructor
  AREXPORT virtual ~ArActionBumpers(void);
  AREXPORT virtual ArActionDesired *fire(ArActionDesired currentDesired);
  AREXPORT virtual ArActionDesired *getDesired(void) { return &myDesired; }
  double findDegreesToTurn(int bumpValue);
protected:
  ArActionDesired myDesired;
  bool mySetMaximums;
  double myBackOffSpeed;
  int myBackOffTime;
  int myTurnTime;
  bool myFiring;
  double mySpeed;
  double myHeading;
  int myBumpMask;
  ArTime myStartBack;
};

#endif // ARACTIONBUMPERS
