/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARARG_H
#define ARARG_H

#include "ariaTypedefs.h"
#include "ariaUtil.h"
#include <string>

/// Argument class, mostly for actions, could be used for other things
/** 
    This is designed to be easy to add another type to the arguments... 
    All you have to do to do so, is add an enum to the Type enum, add a 
    newType getNewType(void), add a void setNewType(newType nt), and add
    a case statement for the newType to ArArg::print.  You should probably also
    add an @see newType to the documentation for ArArg::getType.

*/
class ArArg
{
public:
  typedef enum 
  { 
    INVALID, ///< An invalid argument, the argument wasn't created correctly
    INT, ///< Integer argument
    DOUBLE, ///< Double argument
    STRING, ///< String argument
    BOOL, ///< Boolean argument
    POSE, ///< ArPose argument
  } Type;
  /// Default empty contructor
  AREXPORT ArArg(void);
  /// Constructor for making an integer argument
  AREXPORT ArArg(std::string name, int *pointer, 
		 std::string description = ""); 
  /// Constructor for making a double argument
  AREXPORT ArArg(std::string name, double *pointer,
		 std::string description = ""); 
  /// Constructor for making a string argument
  AREXPORT ArArg(std::string name, std::string *pointer,
		 std::string description = ""); 
  /// Constructor for making a boolean argument
  AREXPORT ArArg(std::string name, bool *pointer,
		 std::string description = ""); 
  /// Constructor for making a position argument
  AREXPORT ArArg(std::string name, ArPose *pointer,
		 std::string description = ""); 
  /// Copy constructor
  AREXPORT ArArg(const ArArg & arg);
  /// Destructor
  AREXPORT virtual ~ArArg(void);
  /// Gets the type of the argument
  AREXPORT Type getType(void);
  /// Gets the name of the argument
  AREXPORT std::string getName(void);
  /// Gets the long description of the argument
  AREXPORT std::string getDescription(void);
  /// Sets the argument value, for int arguments
  AREXPORT void setInt(int val);
  /// Sets the argument value, for double arguments
  AREXPORT void setDouble(double val);
  /// Sets the argument value, for string arguments
  AREXPORT void setString(std::string str);
  /// Sets the argument value, for bool arguments
  AREXPORT void setBool(bool val);
  /// Sets the argument value, for ArPose arguments
  AREXPORT void setPose(ArPose pose);
  /// Gets the argument value, for int arguments
  AREXPORT int getInt(void); 
  /// Gets the argument value, for double arguments
  AREXPORT double getDouble(void);
  /// Gets the argument value, for string arguments
  AREXPORT std::string getString(void);
  /// Gets the argument value, for bool arguments
  AREXPORT bool getBool(void);
  /// Gets the argument value, for pose arguments
  AREXPORT ArPose getPose(void);
  /// Logs the type, name, and value of this argument
  AREXPORT void print(void);
  /// Internal helper function
  AREXPORT void clearPointers(void);
protected:

  ArArg::Type myType;
  std::string myName;
  std::string myDescription;
  int *myIntPointer;
  double *myDoublePointer;
  std::string *myStringPointer;
  bool *myBoolPointer;
  ArPose *myPosePointer;
};

#endif // ARARGUMENT_H
