/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARLOG_H
#define ARLOG_H

#ifdef linux
#include <stdio.h>
#endif
#include <string>
#include "ariaTypedefs.h"
#include "ArMutex.h"


/// Logging utility class
/**
   ArLog is a utility class to log all messages from Aria to a choosen
   destintation. Messages can be logged to stdout, stderr, a file, and
   turned off completely. Logging by default is set to stdout. The level
   of logging can be changed as well. Allowed levels are Terse, Normal,
   and Verbose. By default the level is set to Normal.
*/
class ArLog
{
public:

  typedef enum {
    StdOut, ///< Use stdout for logging
    StdErr, ///< Use stderr for logging
    File, ///< Use a file for logging
    Colbert, ///< Use a Colbert stream for logging
    None ///< Disable logging
  } LogType;
  typedef enum {
    Terse, ///< Use terse logging
    Normal, ///< Use normal logging
    Verbose ///< Use verbose logging
  } LogLevel;

  /// Log a message
  AREXPORT static void log(LogLevel level, char *str, ...);
  /// Initialize the logging utility
  AREXPORT static bool init(LogType type, LogLevel level,
			    const char *fileName="");
  /// Close the logging utility
  AREXPORT static void close();

  // Do not use this unless you know what you are doing...
  AREXPORT static void logNoLock(LogLevel level, char *str, ...);

  // We use this to print to a Colbert stream, if available
  AREXPORT static void (* colbertPrint)(int i, char *str);

protected:

  static ArLog *ourLog;
  static ArMutex ourMutex;
  static LogType ourType;
  static LogLevel ourLevel;
  static FILE *ourFP;
  static int ourColbertStream;
  static std::string ourFileName;
};


#endif
