/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef _ARPERIODICTASK_H
#define _ARPERIODICTASK_H


#include "ariaTypedefs.h"


/// A periodic task base class
/**
   This class is a base class for running periodic tasks that need to be
   run in their own thread. Since the tasks are periodic and dont need to
   be run constantly, they dont need their own thread all the time. Since
   threads can be a restricted resource, there is this periodic task. The
   user simply needs to derive their own class from ArPeriodicTask and
   override the runTask() function. The function is guarenteed to be run
   in its own thread. It can run as long as it wants and when its done
   the thread will be released to be used by another ArPeriodicTask that
   may need it. To run the task, simply call invoke() on it. The invoke()
   can be called multiple times.

   See also ArTaskPool.
*/
class ArPeriodicTask
{
public:

  /// Constructor
  AREXPORT ArPeriodicTask();

  /// Destructor
  AREXPORT virtual ~ArPeriodicTask();

  /// Invoke the task
  AREXPORT virtual bool invoke();

  /// Function to override for the task
  AREXPORT virtual void runTask() = 0;

  /// Function to set the task's running variable
  AREXPORT void setRunning(bool yesno);

  /// Function to tell if the task is running
  AREXPORT bool isRunning() { return myIsRunning; }

protected:

  bool myIsRunning;
  ArMutex myMutex;

};


#endif // _ARPERIODICTASK_H
