/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARRANGEDEVICETHREADED_H
#define ARRANGEDEVICETHREADED_H

#include "ariaTypedefs.h"
#include "ArRangeDevice.h"
#include "ArASyncTask.h"

/// A range device which can run in its own thread
class ArRangeDeviceThreaded : public ArRangeDevice, public ArASyncTask
{
public:
  /// Constructor
  AREXPORT ArRangeDeviceThreaded(size_t currentBufferSize, 
				 size_t cumulativeBufferSize,
				 const char *name, unsigned int maxRange);
  /// Destructor
  AREXPORT virtual ~ArRangeDeviceThreaded(void);
  AREXPORT virtual void * runThread(void *arg) = 0;
  /// Runs the device in this thread
  /**
     Runs the device in the calling thread.
     @see ArThread::stopRunning
  */
  AREXPORT void run(void) { runInThisThread(); }
  /// Runs the device in its own thread
  /**
     Runs the device in its own thread.
     @see ArThread::stopRunning
  */
  AREXPORT void runAsync(void) { create(false); }
  AREXPORT virtual int lockDevice(void) { return lock(); }
  AREXPORT virtual int tryLockDevice(void) { return tryLock(); }
  AREXPORT virtual int unlockDevice(void) { return unlock(); }
  
};

#endif // ARRANGEDEVICETHREADED_H
