/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARROBOTPARAMFILE_H
#define ARROBOTPARAMFILE_H


#include "ariaUtil.h"
#include "ArPreferences.h"


class ArRobotParamFile : public ArPreferences
{
public:

  // Section names
  enum { General, ConvFactors, Accessories, Sonar, Laser, LASTSECTION };

  // Preference names
  enum
  {
    // General parameters
    Class, Subclass, RobotRadius, RobotDiagonal, Holonomic, MaxRVelocity,
    MaxVelocity, HasMoveCommand, RequestIOPackets,

    // Conversion factors
    AngleConvFactor, DistConvFactor, VelConvFactor, RangeConvFactor,
    DiffConvFactor, Vel2Divisor,

    // Accessories
    TableSensingIR, NewTableSensingIR, FrontBumpers, NumFrontBumpers,
    RearBumpers, NumRearBumpers,

    // Sonar parameters
    SonarNum, SonarUnit,

    LaserPossessed, LaserPort, LaserPowerControlled, 
    LaserFlipped, LaserX, LaserY,
    LASTPREF
  };

  static const char *ourClassValues[];
  static const char *ourSubclassValues[];
  static ArPrefSection ourSections[];
  static ArPref ourPrefs[];

  AREXPORT ArRobotParamFile(const char *dir, const char *file);
  AREXPORT virtual ~ArRobotParamFile();

  AREXPORT virtual void setDefaults() = 0;

  AREXPORT virtual void init();

  AREXPORT virtual std::string getHeader();
  AREXPORT virtual std::string getDir();
  AREXPORT virtual std::string getFile();

protected:

  AREXPORT virtual bool findDir(const char *fileName);
  AREXPORT virtual bool findFile(const char *fileName);
  AREXPORT virtual bool handleListSectLine(const char *section,
					   const char *line);
  AREXPORT virtual void extraSave(FILE *fp);

  std::string myDir;
  std::string myFile;
  std::string myTmpFile;

private:

  // Hide the default constructor.
  AREXPORT ArRobotParamFile();

};


#endif // ARROBOTPARAMFILE_H
