/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARSEMAPHORE_H
#define ARSEMAPHORE_H


#ifdef WIN32
#include "winbase.h"
#include "ArMutex.h"
#else
#include <semaphore.h>
#endif
#include <string>
#include "ariaTypedefs.h"

class ArSemaphore
{
public:

#ifdef WIN32
  typedef HANDLE SemaType;
#else
  typedef sem_t SemaType;
#endif

  enum {STATUS_FAILED_INIT, STATUS_TRY_AGAIN, STATUS_FAILED_UNKNOWN};

  AREXPORT ArSemaphore();
  AREXPORT virtual ~ArSemaphore();

  AREXPORT virtual int wait();
  AREXPORT virtual int tryWait();
  AREXPORT virtual int post();
  AREXPORT virtual int getValue(int &value);
  AREXPORT virtual std::string getError(int messageNumber);

protected:

  bool myFailedInit;
  SemaType mySema;
  ArStrMap myStrMap;
#ifdef WIN32
  ArMutex myMutex;
  long myCount;
#endif
};


#endif // ARSEMAPHORE_H
