/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARSERIALCONNECTION_H
#define ARSERIALCONNECTION_H

#include <string>
#include "ariaTypedefs.h"
#include "ArDeviceConnection.h"

#ifdef linux
#define TIOGETTIMESTAMP         0x5480
#define TIOSTARTTIMESTAMP       0x5481
#endif

/// For connecting to devices through a serial port
class ArSerialConnection: public ArDeviceConnection
{
 public:
  /// Constructor
  AREXPORT ArSerialConnection(void);
  /// Destructor also closes the connection
  AREXPORT virtual ~ArSerialConnection(void);

  /// Opens the serial port
  AREXPORT int open(const char * port = NULL);

  /// Sets the port this will use
  AREXPORT void setPort(const char *port = NULL);
  /// Gets the port this is using
  AREXPORT const char * getPort(void);
  
  AREXPORT virtual bool openSimple(void);  
  AREXPORT virtual int getStatus(void);
  AREXPORT virtual bool close(void);
  AREXPORT virtual int read(const char *data, unsigned int size, 
			    unsigned int msWait = 0);
  AREXPORT virtual int write(const char *data, unsigned int size);
  AREXPORT virtual std::string getOpenMessage(int messageNumber);

  /// Sets the baud rate on the connection
  AREXPORT bool setBaud(int baud);
  /// Gets what the current baud rate is set to
  AREXPORT int getBaud(void);

  /// Sets whether to enable or disable the hardware control lines
  AREXPORT bool setHardwareControl(bool hardwareControl);
  /// Gets whether the hardware control lines are enabled or disabled
  AREXPORT bool getHardwareControl(void);

  // Internal open, for use by open and openSimple
  AREXPORT int internalOpen(void);

  enum Open { 
      OPEN_COULD_NOT_OPEN_PORT = 1, ///< Could not open the port
      OPEN_COULD_NOT_SET_UP_PORT, ///< Could not set up the port
      OPEN_INVALID_BAUD_RATE, ///< Baud rate is not valid
      OPEN_COULD_NOT_SET_BAUD, ///< Baud rate valid, but could not set it
      OPEN_ALREADY_OPEN ///< Connection was already open
  };
  AREXPORT virtual ArTime getTimeRead(int index);
  AREXPORT virtual bool isTimeStamping(void);

 protected:
  void buildStrMap(void);

#ifdef linux
  // these both return -1 for errors
  int rateToBaud(int rate);
  int baudToRate(int baud);
  // this just tries
  void startTimeStamping(void);
  bool myTakingTimeStamps;
#endif
    
  ArStrMap myStrMap;
  std::string myPortName;
  int myBaudRate;
  int myStatus;
  bool myHardwareControl;

#ifdef linux
  int myPort;
#endif // ifdef linux

#ifdef WIN32
  HANDLE myPort;
#endif // ifdef WIN32


};

#endif
