/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARSICKLOGGER_H
#define ARSICKLOGGER_H

#include <stdio.h>

#include "ariaUtil.h"
#include "ArFunctor.h"

class ArSick;
class ArRobot;

/// This class can be used to create log files for the laser mapper
/**
   This class has a pointer to a robot and a laser... every time the
   robot has EITHER moved the distDiff, or turned the degDiff, it will
   take the current readings from the laser and log them into the log
   file given as the filename to the constructor.
*/
class ArSickLogger
{
public:
  /// Constructor
  AREXPORT ArSickLogger(ArRobot *robot, ArSick *sick, double distDiff, 
			double degDiff, const char *filename);
  /// Destructor
  AREXPORT virtual ~ArSickLogger(void);
  /// Adds a string to the log file at the given moment
  AREXPORT void addTagToLog(const char *str, ...);
  /// Sets the distance at which the robot will take a new reading
  void setDistDiff(double distDiff) { myDistDiff = ArMath::fabs(distDiff); }
  /// Gets the distance at which the robot will take a new reading
  double getDistDiff(void) { return myDistDiff; }
  /// Sets the degrees to turn at which the robot will take a new reading
  void setDegDiff(double degDiff) { myDistDiff = ArMath::fabs(degDiff); }
  /// Gets the degrees to turn at which the robot will take a new reading
  double getDegDiff(void) { return myDegDiff; }
  /// The task which gets attached to the robot
  AREXPORT void robotTask(void);

protected:
  std::list<std::string> myTags;
  ArRobot *myRobot;
  ArSick *mySick;
  FILE *myFile;
  bool myFirstTaken;
  ArPose myLast;
  double myLastVel;
  double myDistDiff;
  double myDegDiff;
  ArSectors mySectors;
  ArFunctorC<ArSickLogger> myTaskCB;
  int myScanNumber;
  ArTime myStartTime;
};

#endif // ARSICKLOGGER_H
