/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef _ARTASKPOOL_H
#define _ARTASKPOOL_H


#include <list>
#include <set>
#include "ariaTypedefs.h"
#include "ArFunctor.h"
#include "ArTaskPoolTask.h"
#include "ArPeriodicTask.h"
#include "ArMutex.h"
#include "ArSemaphore.h"


/// A thread pool for running functions or ArPeriodicTasks
class ArTaskPool
{
public:

  /// Get a pointer to the 'single' instance of ArTaskPool
  AREXPORT static ArTaskPool * getPool();
  /// Set the pointer to the 'single' instance of ArTaskPool
  AREXPORT static void setPool(ArTaskPool *pool);

  /// Constructor
  AREXPORT ArTaskPool();

  /// Destructor
  AREXPORT virtual ~ArTaskPool();

  /// Run a function as a periodic task
  AREXPORT virtual bool runTask(ArFunctor *task);
  /// Run a periodic task
  AREXPORT virtual bool runTask(ArPeriodicTask *task);

  /// Init the pool.
  AREXPORT virtual void init(int minLimit=1, int maxLimit=0);

  /// Uninit the pool. Blocks till all tasks exit.
  AREXPORT virtual void uninit();

  /// Force all tasks to exit, then uninit the pool.
  AREXPORT virtual void forceUninit();

  AREXPORT virtual bool runTask(std::pair<ArFunctor*,ArPeriodicTask*> task);
  AREXPORT virtual void makeThread(bool makeBusy);
  AREXPORT virtual void reset();

  static ArTaskPool *ourPool;

  bool myRunning;
  int myMaxLimit;
  int myMinLimit;
  std::set<ArTaskPoolTask*> myThreads;
  std::set<ArTaskPoolTask*> myFreeThreads;
  std::set<ArTaskPoolTask*> myBusyThreads;
  std::list<std::pair<ArFunctor*,ArPeriodicTask*> > myTaskQueue;
  ArMutex myMutex;
  ArSemaphore myThreadIdlerSem;
};


#endif // _ARTASKPOOL_H
