//
// SfBandStereoDev.h
//
// Saphira band stereo device class defs

//
// Copyright 2001 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com


#ifndef SFBANDSTEREODEV_H
#define SFBANDSTEREODEV_H

#undef SFEXPORT
#if defined(MS_WINDOWS) && !defined(SF_STATIC)
#ifdef MAKE_LIBRARY
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif


//
// SfBandStereoDevice
// Creates an ArSonarDevice and incorporates it
//

class SfBandStereoDevice : public SfRangeDevice
{
public:
  SFEXPORT SfBandStereoDevice(ArPose spose, int curBufSize, 
			      int maxRange = 5000);
  SFEXPORT ~SfBandStereoDevice();
  // starts up the device with .ini file
  // if fname is present, uses file images; else hardware device
  SFEXPORT bool start(char *ini, char *fname = NULL); 
  SFEXPORT bool stop();		// shuts down the device
  void request() { myRequest = true; }	// request a new reading
  void draw(SfWin *);
  void process();		// what we do 
  void addAccum();		// adds readings into the accum buffer
  void clearAccum();		// clears the accum buffer

protected:			
  // this will get called as a sync task using process(), look at the constructor
  ArFunctorC<SfBandStereoDevice> myBandStereoTask;

  // buffers
  int myCurBufSize, myCumBufSize, myMaxRange;
  std::list<ArSensorReading *> *myRawReadings;
  ArPose mySensorPose;
  void setRawBuffers();		// set raw buffer reading parameters
  int cellWidth, cellHeight;	// band cell size

  // SVS objects
  svsAcquireImages *sourceObject; // source of images
  svsStereoProcess *processObject; // for computing disparity image
  bool calcCellDist(int x, int y, svsStereoImage *si, int *sx, int *sy); // distance to a cell
  void filterReadings();	// filter out long range readings
  double myFilterNearDist;	// how near two readings can be in current buffer
  double myFilterFarDist;	// how far away is too far in current buffer

  // request a new stereo image
  bool myRequest;

};



#endif // SFBANDSTEREODEV_H
