//
// SfConstant.h -- Constants used in the handler
//


//
// SfConstants.h
//
// Copyright 2001 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com
//


#ifndef SFCONSTANTS_H
#define SFCONSTANTS_H

#ifdef WIN32
#define MS_WINDOWS
#else
#define IS_UNIX
#endif

#undef SFEXPORT
#if defined(MS_WINDOWS) && !defined(SF_STATIC)
#ifdef MAKE_LIBRARY
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif


#include <math.h>
#ifdef NOSYS_TYPES
#include <types.h>
#else
#include <sys/types.h>
#endif


//
// some color definitions, from FLTK
//

#define sfColorDarkTurquoise 10
#define sfColorDarkOliveGreen 11
#define sfColorOrangeRed     12
#define sfColorMagenta        5
#define sfColorSteelBlue      4
#define sfColorBrickRed       1
#define sfColorRed            1
#define sfColorLightRed       1
#define sfColorYellow         3
#define sfColorLightYellow    3
#define sfColorBlack          0
#define sfColorWhite          7
#define sfColorGray          49
#define sfColorMediumGray    45 
#define sfColorLightGray     52
#define sfColorGreen          2
#define sfColorCyan           6

#define sfLineSolid           0
#define sfLineDashed          1


// left and right are useful constants
#define sfLEFT  0
#define sfRIGHT 1

#define DEG_TO_RAD (M_PI / 180.0)
#define RAD_TO_DEG (180.0 / M_PI)

// angular units are rads 

#ifdef WIN32
#define M_PI_2 (M_PI / 2.0)
#endif
#define ANG30 (M_PI * 30.0 / 180.0)
#define ANG45 M_PI_4
#define ANG60 (M_PI * 60.0 / 180.0)
#define ANG70 (M_PI * 70.0 / 180.0)
#define ANG180 M_PI
#define ANG240 (M_PI * 240.0 / 180.0)
#define ANG270 (M_PI + M_PI_2)
#define ANG360 (M_PI + M_PI)

// max and min and abs 
#define ANG90 M_PI_2
#define ANG100 (M_PI * 100.0 / 180.0)
#define ANG120 (M_PI * 120.0 / 180.0)
#define ANG135 (ANG90 + ANG45)

#ifndef MAX
#define MAX(a,b) (((a)>(b))?(a):(b))
#endif
#ifndef MIN
#define MIN(a,b) (((a)<(b))?(a):(b))
#endif
#define ABS(x) ((x) > 0.0 ? (x) : (- (x)))
#define SIGNUM(x) ((x) > 0.0 ? 1 : ((x) < 0.0 ? -1 : 0))
#define FSIGNUM(x) ((x) > 0.0 ? 1.0 : ((x) < 0.0 ? -1.0 : 0.0))

// rounding

#define ROUNDPOS(x) ((int)(x + 0.5))
#define ROUNDUP(x) ((int)ceil(x))
#ifndef ROUND
#ifdef LINUX
#define ROUND(x)  ((int)rint(x))
#else
#define ROUND(x)  ((int)floor((x) + 0.5))
#endif
#endif


// shared object fns 
#ifdef WIN32
extern "C" {
void *dlopen(char *fname, int flag);
void dlclose(void *h);
void *dlsym(void *handle, char *sym);
char *dlerror(void);
}
#define RTLD_NOW 0
#define RTLD_GLOBAL 0
#else
#include <dlfcn.h>
#endif
#define RTLD_FLAGS RTLD_NOW | RTLD_GLOBAL

// some directory definitions 
#ifdef WIN32
#define bigint __int64
#define PARAMSTR "%s\\params\\"
#define MAPSTR "%s\\worlds"
#define DIRSEPSTR "\\"
#define DIRSEPCHAR '\\'
#else
#define PARAMSTR "%s/params/"
#define MAPSTR "%s/worlds"
#define DIRSEPSTR "/"
#define DIRSEPCHAR '/'
#endif
#define DIRSEPCHARUNIX '/'


// default directories 
// need to make these into a static class

SFEXPORT extern char *sfParamDir;
SFEXPORT extern char *sfMapDirectory;
SFEXPORT extern char *sfLoadDirectory;
SFEXPORT extern char *sfTopDirectory; // top level Saphira directory 

SFEXPORT extern char *sfComSerial; // tty port name 
SFEXPORT extern int   sfSerialBaud; // tty port baud rate 
SFEXPORT extern char *sfComServer; // TCP/IP server name 
SFEXPORT extern char *sfComPipe; // local transport name 
SFEXPORT extern int   sfComServerPort;	// port number for TCP connection 
SFEXPORT extern int   sfComPortType;	// default connection type 

void shorten(char *name);	/* shorten the file name into outbuf */
extern char namebuf[];

// channels
#define TROBOT_PORT 8101	/* port number by default */
#define UROBOT_PORT 8102	/* port number by default */
#define TCP 0			/* protocol number */
//enum { TCP=0 };
#define TCPPORT 0		/* protocol number */
#define UDP 1			/* protocol number */
//enum   { UDP=1 };
#define UNIXPORT 4		/* protocol number */
#define TTYPORT 5		/* protocol number */
#define CHILDPORT 6		/* child stdin/stdout */
#define LOCALPORT 6
#define NOPORT 7		/* no port, just open */

#ifdef WIN32
#define SERIAL_COM_1 "COM1"
#define SERIAL_COM_2 "COM2"
#else
#define SERIAL_COM_1 "/dev/ttyS0"
#define SERIAL_COM_2 "/dev/ttyS1"
#endif

#define sfTTYPORT TTYPORT       /* channel types */
#define sfLOCALPORT CHILDPORT
#define sfTCPPORT TCPPORT
#define sfCOM1 SERIAL_COM_1     /* channel names */
#define sfCOM2 SERIAL_COM_2
#define sfCOMLOCAL ROBOT_SOCKET

#endif  // SFCONSTANT.H


