//
// SfLogger.h
//
// Logging code 
// Saves/replays a robot data log
// FLTK window code is in SfLoggerFl.cpp
//
// Copyright 2002 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com
//

//
// Log files are a way of saving and playing back actual robot data, as opposed 
//   running the simulator.  Currently only robot motion and sonar data is logged.
// Of course, control commands sent to the robot don't do anything when operating
//   on a log file.  These files are most useful for understanding how sensor data
//   is interpreted.
// It might be useful to save commands, also, so we could check what Saphira/Aria
//   did on the data.
//

#ifndef SFLOGGER_H
#define SFLOGGER_H

#include <stdio.h>
#include "export.h"

class SfLogger
{
 public:
  SFEXPORT SfLogger(SfRobot *robot = SfROBOT, char *fname = NULL);
  SFEXPORT ~SfLogger();
  SFEXPORT void setLogFile(char *fname)
    { fileName = fname; };

  SFEXPORT bool startRecord();  // starts up recording, opens file
  SFEXPORT void stopRecord();   // stops recording and closes file
  SFEXPORT void pauseRecord();  // pauses recording; restart with startRecord()

  SFEXPORT bool startPlay() { return false; };    // starts playback, opens file
  SFEXPORT void stopPlay();     // stops playback and closes file
  SFEXPORT void pausePlay() {};	// pauses playback; restart with startPlay()

 private:
  // main callback fn for logging to a file
  ArRetFunctor1C<bool, SfLogger, ArRobotPacket *> myPacketLoggerCB;
  bool processPacket(ArRobotPacket *packet);
  
  // robot object
  SfRobot *robot;

  // file name, file handle
  char *fileName;
  FILE *fd;

  // recording or playing
  bool recordFlag, playFlag;
};



#endif
