//
// SfMapper.h
//
// mapper definitions

//
// Copyright 2002 by benson limketkai
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// benson limketkai
// Student Associate
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  bensonl@ai.sri.com
//


#ifndef SFMAPPER_H
#define SFMAPPER_H

#include "export.h"
#include "scanstudio/mapper.h" // NOTE: no need extern "C"

extern const long MAPPER_MAX_ROBOTS;

class SfLaserScan;
class asyncLaserScanProcessor;

enum updateType { NOUPDATE=0, SCANSTUDIO=1 };

class SfMapper : public SfUTask
{
  public:
    SFEXPORT SfMapper();
    SFEXPORT void process();
    SFEXPORT void reset();
    std::list<SfLaserScan*>* setOfScans;
    ulong nextScanId;
    struct MapperHandle* MapperHandle; // for ScanStudio
    updateType updateMethod;
    ArSickLogger* logger;
    int robot_id;
    int transmitData;

    SFEXPORT void resetScanStudio();

    SFEXPORT void setMinAngle(double angle)
	{ move_da = angle; }
    SFEXPORT void setMinDistance(double dist)
	{ move_ds = dist; }
    SFEXPORT void setRobotId(int id)
	{ robot_id = id; }
    SFEXPORT void startTransmitData()
	{ transmitData = 1; }
    SFEXPORT void stopTransmitData()
	{ transmitData = 0; }

  private:
    asyncLaserScanProcessor* scanProcessor;
    ArPose lastPose; // for updating
    //ArMutex callbackMutex; BUGBUG: i want this to be STATIC!!!
    ArMutex processingMutex;
    
    double move_ds, move_da;

    // for ScanStudio (see ScanStudio/simu/mapper.c)
    struct PoseProb* PoseProb;
    short PoseProbType;
    double PoseGridXRes, PoseGridYRes, PoseGridARes;
    long PoseSamplesNum;
    ScanMatcher* MapperScanMatcher;
    double MapperMinOverlap;
    short MapperChainLen;
    double MapperMergeRadius;
    int MapperRectilinear;
    double MapperCloseLoopMinProb;
    int MapperCPEMethod;
    
    SFEXPORT static bool mapper_closeloopcallback();
    SFEXPORT static void mapper_redrawcallback(struct Scan **scans, long numScans,
					       struct PoseGauss (*robotPos)[MAPPER_MAX_ROBOTS], bool all);
};


class asyncLaserScanProcessor : public ArRecurrentTask
{
  public:
    SFEXPORT asyncLaserScanProcessor(SfMapper* m);
    SFEXPORT void task();  // async updating task
    
  private:
    SfMapper* mapper;
};


#endif // SFMAPPER_H
