//
// SfMonteLaser.h
//
// monte carlo localization with a laser class definitions

//
// Copyright 1999 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com

#ifndef SFMONTELASER_H
#define SFMONTELASER_H

#include "export.h"

/**
   \defgroup laserNav Laser Localization/Navigation

   The laser Localization/Navigation module is seperate from the
   normal Saphira distribution.  It comes with the complete Laser
   Mapping and Navigation package ActivMedia Robotics.  For
   information contact sales@activmedia.com.

   If you have purchased this module you should be able to download
   new versions of this module from
   http://robots.activmedia.com/navigation

   Do make sure your Saphira/versionNavigation file is the same
   version as your Saphira/version.txt file.  There is dependency
   checking in Linux already, but not yet in the Windows installer.
   Its critical you're using the same versions for Saphira and its
   modules.

   This module allows Saphira to use the the laser for Localization.

   In addition to the documentation here, look at scan.act to see how
   to use these.
**/

/*@{*/

/// Initializes the localization
SFEXPORT void mcLrfInit();
/// Initializes the localization with a particular size of grid
SFEXPORT void mcLrfInitRes(int res);
/// Adds in the cumulative LRF buffer to map
SFEXPORT void mcLrfScanInit();
/// Loads a scan map
SFEXPORT void mcLoadScanMap(char *name);

/*@}*/

//
// Subclass SfCorrSamples -- uses correlation as posterior 
//   probability calculation
// Used with LRF currently, but could be used with sonars,
//   given the grid contains wall angles
//
// For map creation, AddCumulative() puts the current cumulative
//   distribution into the grid
//

class SfCorrSamples : public SfSamples
{
public:
  // number of samples, size of grid, sensor device
  SFEXPORT SfCorrSamples(int num, SfVector &s, SfRangeDevice *dev); 
  SFEXPORT void PostProbInit(); // set up LRF readings
  SFEXPORT void PostProb(SfPoseP *p, int n); // posterior prob calculation
  SFEXPORT virtual void DoMapUpdate(); // adds in current LRF cumulative
  SFEXPORT void AddMapCurrent(); // for map building	

  void SetSensor(SfRangeDevice *sr) // set up which sensor to use
    { sensor = sr; }
  SfRangeDevice *GetSensor() { return sensor; } 

  int dist;			// minimum distance filter

private:
  double *xx, *yy;		// point buffers
  int numreadings;		// how many we have
  SfRangeDevice *sensor;	// which sensor we're using
};

#endif // SFMONTELASER_H
