//
// SfUTask.h
//
// Saphira uTask definitions

//
// Copyright 2001 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com


#ifndef SFUTASK_H
#define SFUTASK_H

#undef SFEXPORT
#if defined(MS_WINDOWS) && !defined(SF_STATIC)
#ifdef MAKE_LIBRARY
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif


//
// The process() member of SfUTask defines the work done by this uTask
//


/// Saphira class that encapsulates the Aria synchronous task
/**
    This Saphira micro-task class is a wrapper for the Aria synchronous task
    facility.  Each SfUTask object is has its process() function called during
    the 100 ms synchronous cycle.

    There are functions for suspending and resuming the uTask, which can be 
    called from within process() or outside of it.

    There is a tutorial program in the directory Saphira/tutor/task.
*/
class SfUTask
{
public:
  /// Constructor, must be chained to by the subclass
  SFEXPORT SfUTask(char *name, int priority);
  /// Destructor
  virtual ~SfUTask() {};
  SFEXPORT void removeTask();	// remove from task list
  SFEXPORT void addTask();      // adds to task list
  /// Suspend the task 
  SFEXPORT void suspendTask();	
  /// Resume the task
  SFEXPORT void resumeTask();
  /// Task main body, called every sync cycle
  virtual void process() {};	// what we do
  /// Process state.  Can be written to during normal processing, to set
  /// a user-defined state.  Should not be used to suspend or resume the
  /// uTask.
  int processState;             // what our state is

private:
  char *myName;                 // what we're called
  int myPriority;               // task priority
  ArFunctor *myFunctor;         // our Aria functor
};

#endif

