//
// fldisp.h
//
// Info window displays
//

//
// Copyright 2000 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com

#ifndef FLDISP_H
#define FLDISP_H

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Multiline_Input.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Float_Input.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Widget.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Menu_Button.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_Browser.H>
#include <FL/Fl_Select_Browser.H>
#include <FL/Fl_File_Chooser.H>


#ifdef MS_WINDOWS
#ifdef SFEXPORT
#undef SFEXPORT
#endif
#ifdef FLTKWIN_EXPORTS
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif

//
// Define a browser window for activities
//
// On creation, pops up a window with a hierarchical activity display
// Runs a process to keep it updated every 2 cycles
//
// Checks activities in the process list
// Note the process list should migrate to the robot object eventually, rather
//  than being global
//


class SfActivityFrame : public Fl_Window
{
 public:
  SFEXPORT SfActivityFrame(int w, int h, bool useTask = true);
  SFEXPORT ~SfActivityFrame();
  void New(int level, int pos, SfActTask *p);
  int NewAll(int level, int pos, SfActTask *p);
  void DeleteAll(int pos);
  void Update(int level, int pos, SfActTask *p);
  int CheckLevel(int level, int pos);
  void Process();

 private:
  Fl_Select_Browser *actBrowser;
};


//
// Define a window for action parameters
//
// On creation, pops up a window with a display of the parameters
// Parameters can be changed
//


class SfActionFrame : public Fl_Window
{
 public:
  SFEXPORT SfActionFrame(SfActTask *a);
  SFEXPORT ~SfActionFrame();

 private:
  SfActTask *act;
};



#endif // FLDISP_H
