//
// flstream.h
//
// Colbert stream class definitions for FLTK windows
// Other Saphira widget classes, e.g., frames
//

//
// Copyright 2000 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com

//
// Classes and functions for manipulating Colbert input/output streams

#ifndef FLSTREAM_H
#define FLSTREAM_H

#ifdef MS_WINDOWS
#ifdef SFEXPORT
#undef SFEXPORT
#endif
#ifdef FLTKWIN_EXPORTS
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif

#undef index     /* this is found in some standard headers */
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Multiline_Input.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Widget.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Menu_Button.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_File_Chooser.H>

//
// SfColbertGraphicStream is the subclass for sending input to Colbert,
//   and displaying output in an FLTK window.
//
// Input from the window is processed by the AddInput member function of
//   SfColbertStream. 
//
// Output to the window comes by a call to the AddOutput member function,
//   which is overridden here.
//

#define COLBERT_PROMPT_SIZE 2
#define COLBERT_PROMPT "> "
// max number of chars in graphics buffer
#define GRAPHIC_BUFFER_MAX 20000

class SfColbertGraphicStream : public SfColbertStream, public Fl_Multiline_Input
{
 public:
  SFEXPORT SfColbertGraphicStream(int x, int y, int w, int h);
  SFEXPORT ~SfColbertGraphicStream();

  SFEXPORT void AddOutput(const char *str); // writes output to msg buffer
  SFEXPORT void AddOutputFlush(); // writes output to the window, only call async
  SFEXPORT void ReplaceInput(char *str); // substitutes a string as the input string
  int OutputPosition() { return output_position; }
  void OutputPosition(int p) { output_position = p; }

 private:
   int output_position;

  // to check key and mouse events
  int handle(int);

  // list of messages to write
  std::list<char *> msgList;
};


#endif // FLSTREAM_H
