#
# OS dependencies
#

OS = $(shell uname -s)
OSVER = $(shell uname -r)
NAMEA = libsf.a
NAMESO = libsf.so.8.0.0
LNAMESO = libsf.so
NAMEXA = libsfx.a
NAMEXSO = libsfx.so.8.0.0
LNAMEXSO = libsfx.so
LIBSO = $(LIBD)$(NAMESO)
LIBXSO = $(LIBD)$(NAMEXSO)
LIBA = $(LIBD)$(NAMEA)
LIBXA = $(LIBD)$(NAMEXA)


ifeq ($(OS),Linux)
CONFIG = LINUX
SRILLIBS = /usr/X11R6/lib/libXm.a /usr/X11R6/lib/libXp.a -lXpm -lXext -lXt -lSM -lICE -lX11 -ldl -lpthread
LLIBS = -lXp -lXpm -lXm -lXext -lXt -lSM -lICE -lX11 -ldl -lpthread
LLIBSX = -ldl -lpthread -Xlinker -Bstatic -lstdc++ -Xlinker -Bdynamic
MOTIFD = /usr/X11R6/
X11D = /usr/X11/
#STATIC = -static
SRISHARED = -shared -E -soname $(NAMESO) # shared library flag
SRISHAREDX = -shared -E -soname $(NAMEXSO) # shared library flag
#SHARED = -shared -E 
SHARED = -shared 
#LD = ld
LD = g++
AR = ar -rs
# used to be a -O2 on the next line
CFLAGS = -g -Wall -D$(CONFIG) -D_REENTRANT 
REENTRANT = -D_REENTRANT
CC = g++
endif

ifeq (FreeBSD,$(OS))    		# FreeBSD
CONFIG = FreeBSD
SRILLIBS = /usr/lib/libXm.a -lXext -lXt -lSM -lICE -lX11
LLIBS = -lXm -lXext -lXt -lSM -lICE -lX11 -lpthread
LLIBSX = -lpthread
MOTIFD = /usr/X11R6/
X11D = /usr/include/X11/
SRISHARED = -Bshareable -soname=$(NAMESO) # shared library flag
SRISHAREDX = -Bshareable -soname=$(NAMEXSO) # shared library flag
SHARED = -Bshareable 
PICFLAG = -fpic
LD = ld
CFLAGS = -g -O -Wall $(PICFLAG) -D$(CONFIG)
CC = gcc
endif

ifeq (NetBSD,$(OS))    		# NetBSD
CONFIG = NetBSD
SRILLIBS = /usr1/local/lib/libXm.a -lXext -lXt -lSM -lICE -lX11
LLIBS = -lXm -lXext -lXt -lSM -lICE -lX11 -lpthread
LLIBSX = -lpthread
MOTIFD = /usr/X11R6/
X11D = /usr1/local/
SRISHARED = -Bshareable -soname=$(NAMESO) # shared library flag
SRISHAREDX = -Bshareable -soname=$(NAMEXSO) # shared library flag
SHARED = -Bshareable 
PICFLAG = -fpic
LD = ld
CFLAGS = -g -O -Wall $(PICFLAG) -D$(CONFIG)
CC = gcc
endif


ifeq (SunOS,$(OS))			# SunOs	
ifeq (4,$(patsubst 4.%,4,$(OSVER)))	# SunOs 4.x
CONFIG = SUN
SRILLIBS = $(MOTIFD)lib/libXm.a -lXext -lXt -lX11 -ldl
# libXt.so seems to have trouble with XtAppInitialize
LLIBS = -lXm -lXext /usr/opt/Motif/released/lib/libXt.a -lX11 -ldl -lpthread
LLIBSX = -ldl -lpthread
MOTIFD = /usr/opt/Motif/released/
# X11D = /usr/opt/X11/released/released/
CFLAGS = -g -O -Wall -D$(CONFIG)
CC = gcc
LD = ld
# STATIC = -Bstatic
SRISHARED = -Bdynamic # shared library flag
SHARED = -Bdynamic # shared library flag
PICFLAG = -fPIC # position-independent code, req'd for shared
endif
ifeq (5,$(patsubst 5.%,5,$(OSVER)))	# SunOs 5.x
CONFIG = SOLARIS
SRILLIBS =  /home/labrea1/quintus/quintus3.2/bin3.2/sun4-5/libXm.a -lXext -lXt -lX11 -lnsl -lsocket -lgen -ldl -lpthread -lposix4
LLIBS =  -lXm -lXext -lXt -lX11 -lnsl -lsocket -lgen -ldl -lpthread -lposix4
LLIBSX = -lnsl -lsocket -ldl -lpthread -lposix4
MOTIFD = /usr/dt/
X11D = /opt/openwin/released/
# STATIC = -B static
SHARED = -G # shared library flag
SRISHARED = -G # shared library flag
SRISHAREDX = -G # shared library flag
PICFLAG = -fPIC # position-independent code, req'd for shared
LD = ld
CFLAGS = -g -O -Wall -D$(CONFIG) -D_REENTRANT $(PICFLAG)
REENTRANT = -D_REENTRANT
CC = gcc
endif
endif

ifeq ($(OS),IRIX)	# SGI IRIX
CONFIG = SGI
SRILLIBS = /usr/irix4/usr/lib/libXm.a -lXt -lX11 -lgen
LLIBS = -lXm -lXt -lX11 -lgen
LLIBSX = -ldl
MOTIFD = /usr/
X11D = /usr/include/X11/
SRISHARED = -shared -all -soname $(NAMESO) # shared library flag
SHARED = -shared -all # shared library flag
# PICFLAG = -fPIC # position-independent code, req'd for shared
CFLAGS = -g -fullwarn -D$(CONFIG)
NORANLIB = 1
CC = cc
LD = ld
endif

ifeq ($(OS),IRIX64)	# SGI IRIX 6.x
CONFIG = SGI
SRILLIBS = /usr/lib/libXm.a -lXt -lX11 -lgen
LLIBS = -lXm -lXt -lX11 -lgen
LLIBSX = -ldl
MOTIFD = /usr/
X11D = /usr/include/X11/
SRISHARED = -shared -all -soname $(NAMESO) # shared library flag
SHARED = -shared -all # shared library flag
PICFLAG = -fPIC # position-independent code, req'd for shared
LD = ld
CFLAGS = -g -O -Wall -D$(CONFIG)
CC = cc
endif

ifeq ($(OS),AIX)	# IBM AIX
CONFIG = AIX
SRILLIBS = libXm.a -lXt -lX11
LLIBS = -lXm -lXt -lX11
LLIBSX = -ldl
MOTIFD = /usr/share/local/motif/usr/
X11D = /usr/include/X11/
LD = ld
CFLAGS = -g -O -Wall -D$(CONFIG)
CC = gcc
endif

ifeq ($(OS),OSF1)	# DEC OSF1
CONFIG = DEC
SRILLIBS =  /usr/lib/libXm.a -lXext -lXt -lX11 -lSM -lICE -ldnet_stub
LLIBS =  -lXm -lXext -lXt -lX11 -lSM -lICE -ldnet_stub
LLIBSX = -lSM -lICE -ldnet_stub
MOTIFD = /usr/
X11D = /usr/
SRISHARED = -shared -expect_unresolved "*" # shared library flag
SHARED = -shared -expect_unresolved "*" # shared library flag
LD = ld
CFLAGS = -g3 -O -D$(CONFIG)
CC = cc -oldc
endif

