//
// test load file for dynamic loading
//

//
// Copyright 2001 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com
//

#include <string>
#include "Saphira.h"
#include "export.h"		// need this for MSW, for def of SFEXPORT
#include <math.h>

// global integer variables
// on some systems, these will not be initialized on load...
int nopen = 0;
int globalvar = 1;

// string variable
char buf[80];
char *mystring;			// we need to set aside space explicitly for a string ptr 


//
// C++ global functions
// Set up functions to create and manipulate a point artifact
// This point will draw on the screen
//

SfPoint *mypoint;

int				// sets up a point artifact
myfn(int a)
{
  sfMessage("Argument to myfn is: %d",a);
  mypoint = new SfPoint(a,0,0);
  return a+2;
}

float
myErf(float a)
{
#ifndef WIN32
  return (float)((erf(((double)a) / sqrt(2.0)) + 1) / 2.0);
#else
  return 0.0;
#endif
}

void				// prints values of a point artifact
showpoint(void *p)
{
  SfPoint *pt = (SfPoint *)p;
  sfMessage("Point x: %d  y: %d  th: %d", 
	     (int)pt->p.getX(), (int)pt->p.getY(), (int)pt->p.getTh());
}

void				// sets the X and Y values of a point
setpoint(void *p, int x, int y)
{
  SfPoint *pt = (SfPoint *)p;
  pt->p.setX((double)x);
  pt->p.setY((double)y);
  sfMessage("Point x: %d  y: %d  th: %d", 
	     (int)pt->p.getX(), (int)pt->p.getY(), (int)pt->p.getTh());
}


//
// C++ global functions
// Set up an interface to the sonar buffers
//

int
get_sonar_dist(int begang, int endang, int *retang)
{
  SfSonarDevice *sd = Sf::sonar(); // get the device
  if (!sd) 
    return 100000;		// large value, no return
  double ret;
  ArPose rpose = SfROBOT->getPose();
  // Aria occupancy function on the sonar current reading buffer
  double fdist = sd->getCurrent()->getClosestPolar((double)begang, (double)endang, 
						rpose, 100000, &ret);
  if (retang)
    *retang = (int)ret;
  return (int)fdist;
}

// try different sonar routines

void
get_sonar_dist_closest(int begang, int endang)
{
  double retang;
  int ret;
  ret = SfROBOT->getClosestSonarRange(begang, endang);
  sfMessage("Dist getClosestSonarRange: %d", (int)ret);
  ret = SfROBOT->checkRangeDevicesCurrentPolar(begang, endang, &retang);
  sfMessage("Dist checkRangeDevicesCurrentPolar: %d %d", (int)ret, (int)retang);
  SfSonarDevice *sd = Sf::sonar(); // get the device
  if (!sd) return;
  ArPose rpose = SfROBOT->getPose();
  // Aria occupancy function on the sonar current reading buffer
  ret = sd->getCurrent()->getClosestPolar((double)begang, (double)endang, 
						rpose, 100000, &retang);
  sfMessage("Dist getClosestPolar: %d %d", (int)ret, (int)retang);
}


//
// Motor stalls from SfROBOT
//

int
sfStalledMotor(int which)
{
  if (which == sfLEFT)
    return (int)SfROBOT->isLeftMotorStalled();
  else
    return (int)SfROBOT->isRightMotorStalled();
}

//
// Command to stop the robot and then let behavioral actions go
//

void
sfResetMotion()
{
  SfROBOT->stop();		// stops robot immediately
  SfROBOT->clearDirectMotion();	// lets behavioral actions through
}


//
// sfLoadInit and sfLoadExit must be declared as exported fns
// under MS Windows
//


SFEXPORT void
sfLoadInit(void)		// this function is evaluated when the object file is loaded
{
  sfMessage("Opened: %d", nopen); // on open, we should get a message
  printf("Opened!\n");		// let the console know too...

  // define constants and variables
  //  sfAddEvalConst("sfLEFT",  sfINT, 0);
  sfAddEvalVar("gv", sfINT, (fvalue *)&globalvar); // note indirection in the last argument!
  sfAddEvalVar("mypoint", sfPTR, (fvalue *)&mypoint); // note indirection in the last argument!
  mystring = buf;		// set up string ptr 
  mystring[0] = 'A';
  mystring[1] = 0;
  sfAddEvalVar("mystring", sfSTRING, (fvalue *)&mystring); // note the indirection in the last argument!


  // define functions
  // any C++ global functions will give an error here if overloaded
  sfAddEvalFn("myfn", (void *)myfn, sfINT, 1, sfINT);
  sfAddEvalFn("myErf", (void *)myErf, sfFLOAT, 1, sfFLOAT);
  sfAddEvalFn("showpoint", (void *)showpoint, sfVOID, 1, sfPTR);
  sfAddEvalFn("setpoint", (void *)setpoint, sfVOID, 3, sfPTR, sfINT, sfINT);
  sfAddEvalFn("get_sonar_dist", (void *)get_sonar_dist, sfINT, 3, sfINT, 
	      sfINT, sfTypeRef(sfINT));	// last arg is pointer to int
  sfAddEvalFn("get_sonar_dist_closest", (void *)get_sonar_dist_closest, sfVOID, 
              2, sfINT, sfINT);
  sfAddEvalFn("sfResetMotion", (void *)sfResetMotion, sfVOID, 0);

  // set up motor stall interface
  sfAddEvalFn("sfStalledMotor", (void *)sfStalledMotor, sfINT, 1, sfINT);
}


SFEXPORT void
sfLoadExit(void)		/* this should be evaluated on unload */
{
  sfMessage("Unloading: %d", nopen);
}



