/** Copyright 1999 by ActivMedia Robotics, LLC
 ** 
 ** ActivMedia Robotics hereby grants to other individuals or organizations
 ** permission to use this software for non-commercial educational
 ** use only.  This software may not be distributed to others
 ** except by ActivMedia Robotics, under the conditions above.
 **
 ** Other than these cases, no part of this software may be used or
 ** distributed without written permission of ActivMedia Robotics.
 **
 ** ActivMedia Robotics does make any representations about the suitability
 ** of this software for any purpose.  It is provided "as is" without
 ** express or implied warranty.
 **
 **/

#include "Saphira.h"
#include "SaphiraACTSClient.h"

SaphiraACTSClient::SaphiraACTSClient()
{
  comm=new SocketClientComm(ACTS_SOCK_DATA);
}

SaphiraACTSClient::~SaphiraACTSClient()
{
  delete comm;
}

#ifndef WIN32
bool SaphiraACTSClient::openPort(char * service,char * hostname=NULL)
#else
bool SaphiraACTSClient::openPort(char * service,char * hostname)
#endif
{
  if (NULL==hostname) {
    return (comm->openPort(service,"localhost"));
  } else {
    return (comm->openPort(service,hostname));
  }
}

bool SaphiraACTSClient::closePort()
{
  return (comm->closePort());
}

bool SaphiraACTSClient::requestPacket()
{
  return comm->sendRequest('0');
}

bool SaphiraACTSClient::requestQuit()
{
  return comm->sendRequest('1');
}

bool SaphiraACTSClient::receiveBlobInfo()
{
  int index=0;
  int numblobs=0;
  int i;  //Iterator variable

  // Wait until the header has been received
  if (!comm->receiveData(NUMCHAN*4)) {
    // The port is down... it's time to exit
    cout << "Problems receiving blob stats" << endl;
    return false;
  }
  //  cout << "Got the header" << endl;
  // Now, determine how many blobs there are
  data=comm->viewData();
  for (i=0;i<NUMCHAN;i++) {
    //    cout << int(data[index]) << " ";
    blob_index[i]=(*(data++))-1;
    blob_index[i] = blob_index[i] << 6;
    blob_index[i]|=(*(data++))-1;

    blob_num[i]=(*(data++))-1;
    blob_num[i] = blob_num[i] << 6;
    blob_num[i]|=(*(data++))-1;

    numblobs+=blob_num[i];
    //    cout << int(data[index]) << " ";
    index++;
  }
  //  cout << endl;
  //  cout << "Parsed the header: numblobs = " << numblobs << endl;
  // Now, get the rest of the data plus the endof character value
  if (!numblobs) {
    cout << "No blobs" << endl;
    return true;
  }
  if (!comm->receiveData(numblobs*BLOB_DATA_SIZE)) {
    // The port is down... it's time to exit
    cout << "Problems receiving blob data" << endl;
    return false;
  }
  // Load up the blob data now
  data=comm->viewData();
  //  cout << "Size of data = " << strlen((char*)data)+(NUMCHAN*2) << endl;
  return true;
}

// How many blobs have been returned in this channel?
int SaphiraACTSClient::numBlob(int channel) const
{
  if ((channel>=0) && (channel <NUMCHAN)) {
    return blob_num[channel];
  } else {
    return -1;
  }
}

// Debugging printout
bool SaphiraACTSClient::viewBlob(int channel,int index)
{
  unsigned char * curbinfo;

  int dataindex;
  int area=0;
  int xcg=0;
  int ycg=0;
  int left=0;
  int right=0;
  int top=0;
  int bottom=0;
  int i;

  if ((channel < 0) || (channel>=NUMCHAN)) {
    return false;
  }
  if ((index < 0) || (index >=blob_num[channel])) {
    return false;
  }
  // Compute the appropriate index into the blob frame
  // The first character of the first blob is found at:
  dataindex = (blob_index[channel]+index)*BLOB_DATA_SIZE;
  curbinfo = &data[dataindex];
  
  // First thing, calculate the area
  area=0;
  for (i=0;i<4;i++) {
    area = area << 6;
    area|= (*(curbinfo++))-1;
  }

  xcg = (*(curbinfo++))-1;
  xcg = xcg << 6;
  xcg |= (*(curbinfo++))-1;

  ycg = (*(curbinfo++))-1;
  ycg = ycg << 6;
  ycg |= (*(curbinfo++))-1;

  left = (*(curbinfo++))-1;
  left = left << 6;
  left |= (*(curbinfo++))-1;

  right = (*(curbinfo++))-1;
  right = right << 6;
  right |= (*(curbinfo++))-1;

  top = (*(curbinfo++))-1;
  top = top << 6;
  top |= (*(curbinfo++))-1;

  bottom = (*(curbinfo++))-1;
  bottom = bottom << 6;
  bottom |= (*(curbinfo++))-1;

#ifdef WIN32
  // The windows version of acts_robot can only show data via sfMessage
  sfMessage("Channel:%d blob:%d a=%d x=%d y=%d l=%d r=%d t=%d b=%d",
		channel+1,index+1,area,xcg,ycg,left,right,top,bottom);
#endif
  // Otherwise, the unix version can just write to the console
  cout << "Channel:" << channel+1 << " blob:" << index+1;
  cout << " a=" << area;
  cout << " x=" << xcg;
  cout << " y=" << ycg;
  cout << " l=" << left;
  cout << " r=" << right;
  cout << " t=" << top;
  cout << " b=" << bottom;
  cout << endl;
  return true;
}

// Fills up a blob structure and returns it
bool SaphiraACTSClient::getBlob(int channel, int index, blob * b)
{  
  int dataindex;
  unsigned char * curbinfo;
  int i;

  if ((channel < 0) || (channel>=NUMCHAN)) {
    return false;
  }
  if ((index < 0) || (index >=blob_num[channel])) {
    return false;
  }
  // Compute the appropriate index into the blob frame
  // The first character of the first blob is found at:
  dataindex = (blob_index[channel]+index)*BLOB_DATA_SIZE;
  curbinfo = &data[dataindex];
  
  // First thing, calculate the area
  b->area=0;
  for (i=0;i<4;i++) {
    b->area = b->area << 6;
    b->area|= (*(curbinfo++))-1;
  }

  b->xcg = (*(curbinfo++))-1;
  b->xcg = b->xcg << 6;
  b->xcg |= (*(curbinfo++))-1;

  b->ycg = (*(curbinfo++))-1;
  b->ycg = b->ycg << 6;
  b->ycg |= (*(curbinfo++))-1;

  b->left = (*(curbinfo++))-1;
  b->left = b->left << 6;
  b->left |= (*(curbinfo++))-1;

  b->right = (*(curbinfo++))-1;
  b->right = b->right << 6;
  b->right |= (*(curbinfo++))-1;

  b->top = (*(curbinfo++))-1;
  b->top = b->top << 6;
  b->top |= (*(curbinfo++))-1;

  b->bottom = (*(curbinfo++))-1;
  b->bottom = b->bottom << 6;
  b->bottom |= (*(curbinfo++))-1;

  return true;
}
