/** Copyright 1999 by ActivMedia Robotics, LLC
 ** 
 ** ActivMedia Robotics hereby grants to other individuals or organizations
 ** permission to use this software for non-commercial educational
 ** use only.  This software may not be distributed to others
 ** except by ActivMedia Robotics, under the conditions above.
 **
 ** Other than these cases, no part of this software may be used or
 ** distributed without written permission of ActivMedia Robotics.
 **
 ** ActivMedia Robotics does make any representations about the suitability
 ** of this software for any purpose.  It is provided "as is" without
 ** express or implied warranty.
 **
 **/


#ifndef _SocketClientComm
#define _SocketClientComm

#ifndef WIN32
#include <arpa/inet.h>
#include <errno.h>
#include <fcntl.h>
#include <iostream.h>
#include <limits.h>
#include <netdb.h>
#include <netinet/in.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>
#include <unistd.h>
#else
#include <errno.h>
#include <iostream.h>
#include <limits.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#endif


/** 
 * Handles socket communications.  This class is designed to run as a 
 * Client in which a client program will ping it and receive information
 * back.  
 */
class SocketClientComm {
  /// Check the file descriptor for new data
  int retval;
  /// Socket port descriptor
  int sock;
  /// Socket descriptor
#ifdef IS_UNIX                 // mrt  1/21/00
  struct sockaddr_un fsaun;
#endif
  /// Placeholder for 'accept' command
  unsigned int fromlen;
  /// Receive buffer
  unsigned char * buf;
  /// Are we connected to a client?
  bool is_connected;
  /// Size of the incoming buffer
  int maxsize;
  /// Search services for a valid port name
  int a2Port(char * service,char *proto);
  /// Converts ascii text to in_addr struct.
  struct in_addr * a2Addr(char *address);
  /// Functions like the read() system call, but better
  int sockRead(unsigned char * buf, size_t count);
  /// Functions like the write() system call, but better
  int sockWrite(const char * buf, size_t count);

public:
  /// Constructor
  SocketClientComm(int size);
  /// Destructor
  ~SocketClientComm();
  /// Returns whether the port is connected
  bool isConnected() const;
  /// Opens a socket
  bool openPort(char * service, char * netaddress);
  /// Closes the connection to a port.
  bool closePort();
  /// Sends a request for data
  bool sendRequest(char req);
  /// Receives the data from the request
  bool receiveData(int buffsize);
  /// Return a pointer to the data
  unsigned char * viewData() const;
};

#endif
