/** Copyright 1999 by ActivMedia Robotics, LLC
 ** 
 ** ActivMedia Robotics hereby grants to other individuals or organizations
 ** permission to use this software for non-commercial educational
 ** use only.  This software may not be distributed to others
 ** except by ActivMedia Robotics, under the conditions above.
 **
 ** Other than these cases, no part of this software may be used or
 ** distributed without written permission of ActivMedia Robotics.
 **
 ** ActivMedia Robotics does make any representations about the suitability
 ** of this software for any purpose.  It is provided "as is" without
 ** express or implied warranty.
 **
 **/

#include <strstream>
#include <iostream.h>
#include "WSocketClientComm.h"

/// Constructor
SocketClientComm::SocketClientComm(int size)
{
	buffer_size=size;
	buffer = new char[buffer_size];

    WSAData wsaData;
	int nCode;
    if ((nCode = WSAStartup(MAKEWORD(1, 1), &wsaData)) != 0) {
		cerr << "WSAStartup() returned error code " << nCode << "." <<
				endl;
    }
}

/// Destructor
SocketClientComm::~SocketClientComm() 
{
	ShutdownConnection(the_socket);
	delete [] buffer;
    WSACleanup();
}

/// Opens a socket
bool SocketClientComm::openPort(char * service, char * netaddress)
{
	int nPort = atoi(service);
    // Find the server's address
    //cout << "Looking up address..." << flush;
    u_long nRemoteAddress = LookupAddress(netaddress);
    if (nRemoteAddress == INADDR_NONE) {
        cerr << endl << "Error looking up address" << endl;
        return false;
    }
    in_addr Address;
    memcpy(&Address, &nRemoteAddress, sizeof(u_long)); 
    //cout << inet_ntoa(Address) << ":" << nPort << endl; 

    // Connect to the server
    //cout << "Connecting to remote host..." << flush;
    the_socket = EstablishConnection(nRemoteAddress, htons(nPort));
    if (the_socket == INVALID_SOCKET) {
        cerr << endl << "Error connecting to server: Invalid Socket" << endl;
        return false;
    }
    //cout << "connected, socket " << the_socket << "." << endl;

	return true;
}

/// Closes the connection to a port.
bool SocketClientComm::closePort()
{
    // Shut connection down
    cout << "Shutting connection down..." << flush;
    if (ShutdownConnection(the_socket)) {
        cout << "Connection is down." << endl;
		return true;
    } else {
        cerr << endl << "Error shutting down the connection" <<
                endl;
		return false;
    }
}

/// Sends a request for data
bool SocketClientComm::sendRequest(char req)
{
    // Send the string to the server
    if (send(the_socket, &req, 1, 0) != SOCKET_ERROR) {
		return true;
	} else {
        return false;
    }
}

/// Receives the data from the request
bool SocketClientComm::receiveData(int data_size)
{
    // Read reply from server
    int nTotalBytes = 0;
	int nNewBytes;
    while (nTotalBytes < data_size) {
        nNewBytes = recv(the_socket, buffer + nTotalBytes, data_size - nTotalBytes, 0);
        if (nNewBytes == SOCKET_ERROR) {
            return false;
        } else if (nNewBytes == 0) {
            cerr << "Connection closed by peer." << endl;
            return false;
        }
        nTotalBytes += nNewBytes;
    }
	return true;
}

/// Return a pointer to the data
unsigned char * SocketClientComm::viewData() const
{
	return (unsigned char*)buffer;
}

//// LookupAddress /////////////////////////////////////////////////////
// Given an address string, determine if it's a dotted-quad IP address
// or a domain address.  If the latter, ask DNS to resolve it.  In
// either case, return resolved IP address.  If we fail, we return
// INADDR_NONE.

u_long SocketClientComm::LookupAddress(const char* pcHost)
{
    u_long nRemoteAddr = inet_addr(pcHost);
    if (nRemoteAddr == INADDR_NONE) {
        // pcHost isn't a dotted IP, so resolve it through DNS
        hostent* pHE = gethostbyname(pcHost);
        if (pHE == 0) {
            return INADDR_NONE;
        }
        nRemoteAddr = *((u_long*)pHE->h_addr_list[0]);
    }

    return nRemoteAddr;
}


//// EstablishConnection ///////////////////////////////////////////////
// Connects to a given address, on a given port, both of which must be
// in network byte order.  Returns newly-connected socket if we succeed,
// or INVALID_SOCKET if we fail.

SOCKET SocketClientComm::EstablishConnection(u_long nRemoteAddr, u_short nPort)
{
    // Create a stream socket
    SOCKET sd = socket(AF_INET, SOCK_STREAM, 0);
    if (sd != INVALID_SOCKET) {
        sockaddr_in sinRemote;
        sinRemote.sin_family = AF_INET;
        sinRemote.sin_addr.s_addr = nRemoteAddr;
        sinRemote.sin_port = nPort;
        if (connect(sd, (sockaddr*)&sinRemote, sizeof(sockaddr_in)) ==
                SOCKET_ERROR) {
            sd = INVALID_SOCKET;
        }
    }
    return sd;
}

//// ShutdownConnection ////////////////////////////////////////////////
// Gracefully shuts the connection sd down.  Returns true if we're
// successful, false otherwise.

bool SocketClientComm::ShutdownConnection(SOCKET sd)
{
    // Disallow any further data sends.  This will tell the other side
    // that we want to go away now.  If we skip this step, we don't
    // shut the connection down nicely.
    if (shutdown(sd, SD_SEND) == SOCKET_ERROR) {
        return false;
    }

    // Receive any extra data still sitting on the socket.  After all
    // data is received, this call will block until the remote host
    // acknowledges the TCP control packet sent by the shutdown above.
    // Then we'll get a 0 back from recv, signalling that the remote
    // host has closed its side of the connection.
    while (1) {
        int nNewBytes = recv(sd, buffer, buffer_size, 0);
        if (nNewBytes == SOCKET_ERROR) {
            return false;
        } else if (nNewBytes != 0) {
            cerr << endl << "FYI, received " << nNewBytes <<
                    " unexpected bytes during shutdown." << endl;
        } else {
            // Okay, we're done!
            break;
        }
    }

    // Close the socket.
    if (closesocket(sd) == SOCKET_ERROR) {
        return false;
    }

    return true;
}

