/** Copyright 1999 by ActivMedia Robotics, LLC
 ** 
 ** ActivMedia Robotics hereby grants to other individuals or organizations
 ** permission to use this software for non-commercial educational
 ** use only.  This software may not be distributed to others
 ** except by ActivMedia Robotics, under the conditions above.
 **
 ** Other than these cases, no part of this software may be used or
 ** distributed without written permission of ActivMedia Robotics.
 **
 ** ActivMedia Robotics does make any representations about the suitability
 ** of this software for any purpose.  It is provided "as is" without
 ** express or implied warranty.
 **
 **/


#ifndef _SocketClientComm
#define _SocketClientComm

#include <algorithm>
#include <winsock.h>

using namespace std;

#if !defined(_WINSOCK2API_) 
// Winsock 2 header defines this, but Winsock 1.1 header doesn't.  In
// the interest of not requiring the Winsock 2 SDK which we don't really
// need, we'll just define this one constant ourselves.
#define SD_SEND 1
#endif

/** 
 * Handles socket communications.  This class is designed to run as a 
 * Client in which a client program will ping it and receive information
 * back.  
 */
class SocketClientComm {
	/// Receive buffer
	char * buffer;
	/// Size of the buffer
	int buffer_size;

	/// The socket
	SOCKET the_socket;

	/// Helper function to look up an address
	u_long LookupAddress(const char* pcHost);

	/// Make a working connection
	SOCKET EstablishConnection(u_long nRemoteAddr, u_short nPort);

	/// Nicely shut down the connection
	bool ShutdownConnection(SOCKET sd);

public:
  /// Constructor
  SocketClientComm(int size);
  /// Destructor
  ~SocketClientComm();
  /// Opens a socket
  bool openPort(char * service, char * netaddress);
  /// Closes the connection to a port.
  bool closePort();
  /// Sends a request for data
  bool sendRequest(char req);
  /// Receives the data from the request
  bool receiveData(int data_size);
  /// Return a pointer to the data
  unsigned char * viewData() const;
};

#endif
