//
// task.cpp
//
// tutorial sample for utasks and asynchronous tasks
//

//
// Copyright 2001 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com


//
// For checking the asynchronous task:
//   Use Init(n) to set up and start the async task
//     the task will print integers up to n, one per second
//   Use Restart() to restart the async task, with a new value
//   Use Reset() to stop the async task
//
// For checking the utask:
//   Use Start() to start up the utask
//   The utask will initiate the the async task, and keep checking
//     to see if it has completed.  When it does, the utask will 
//     print a message and exit.
//

#include "Saphira.h"
#include "export.h"
 
//
// myAsyncTask -- subclass of ArRecurrentTask
// Just run for N seconds, printing out a message each second
//

class myAsyncTask : public ArRecurrentTask
{
 public:
  SFEXPORT myAsyncTask(int n);	// constructor
  SFEXPORT void task();		// async updating task
  int counter;
  int end;
};


//
// uTask definition, subclass of SfUTask
// 

class myProcess : public SfUTask
{
public:
  myProcess();
  void process();		// this is the workhorse, called every 100 ms
  int counter;
};

//
// Async task member fns
//

// set up counter

myAsyncTask::myAsyncTask(int n)
  : ArRecurrentTask()
{
  counter = 0;
  end = n;
}


// instance of the Async task
myAsyncTask *mytask = NULL;

// instance of the uTask
myProcess *myproc = NULL;


//
// Update processing
// Runs asynchronously
//

void myAsyncTask::task()	// main task we are doing
{				// print N times, once per second
  while (counter <= end)
    {
      sfMessage("Async task counter: %d", counter++);
      ArUtil::sleep(1000);
      lock();			// lock out the sync loop, so we can change values
      if (myproc)
        myproc->counter = 0;
      unlock();			// unlock and go
    }
  sfMessage("Async computation finished");
}

// Interface to Colbert

void Init(int n)		// sets up an instance of the async task
{
  mytask = new myAsyncTask(n);
  mytask->go();
}

void Restart(int n)		// restarts the async task
{
  if (mytask)
    {
      mytask->counter = 0;
      mytask->end = n;
      mytask->go();
    }
}

void Reset(void)		// resets (and stops) the async task
{
  if (mytask)
    mytask->reset();
}
      
int Done(void)			// check if the async task is done
{
  if (mytask)
    return mytask->done();
  else
    return -1;
}


void Start()			// sets up an instance of the uTask
{
  myproc = new myProcess();
}

void Resume()			// resumes the uTask after it stops itself
{
  if (myproc)
    myproc->resumeTask();
}



//
// uTask member fns
//
    
myProcess::myProcess()
  : SfUTask("My Process", 10)	// have to chain to the basic uTask object
{
  Init(10);			// start up the async process to run for 20 seconds
  counter = 0;
}

void
myProcess::process()
{
  if (Done())
    {
      sfMessage("Async task done, suspending uTask");
      suspendTask();
    }
  else
    sfMessage("uTask at %d", counter++);
}


SFEXPORT void			// define interface to Colbert here 
sfLoadInit (void)
{
  // calls to the synchronous utask
  sfAddEvalFn("Start", (void *)Start, sfVOID, 0);

  // calls to the asynchronous task
  sfAddEvalFn("Init", (void *)Init, sfVOID, 1, sfINT);
  sfAddEvalFn("Restart", (void *)Restart, sfVOID, 1, sfINT);
  sfAddEvalFn("Reset", (void *)Reset, sfVOID, 0);
  sfAddEvalFn("Done", (void *)Done, sfINT, 0);
  sfAddEvalFn("Resume", (void *)Resume, sfVOID, 0);
}

